@@ -18,8 +18,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Main application
@@ -44,7 +44,6 @@ from pitivi.effects import EffectsHandler
 from pitivi.configure import APPNAME
 from pitivi.settings import GlobalSettings
 from pitivi.threads import ThreadMaster
-from pitivi.pluginmanager import PluginManager
 from pitivi.signalinterface import Signallable
 from pitivi.log.loggable import Loggable
 from pitivi.log import log
@@ -64,6 +63,7 @@ from pitivi.ui.startupwizard import StartUpWizard
 # AND THEN load up the required parts.
 # This will result in a much better end-user experience
 
+
 class Pitivi(Loggable, Signallable):
     """
     Pitivi's main application class.
@@ -85,8 +85,6 @@ class Pitivi(Loggable, Signallable):
 
     @ivar settings: Application-wide settings.
     @type settings: L{GlobalSettings}.
-    @ivar projects: List of used projects
-    @type projects: List of L{Project}.
     @ivar current: Currently used project.
     @type current: L{Project}.
     """
@@ -121,7 +119,6 @@ class Pitivi(Loggable, Signallable):
                 % APPNAME)
         instance.PiTiVi = self
 
-        self.projects = []
         self.current = None
 
         # get settings
@@ -129,11 +126,7 @@ class Pitivi(Loggable, Signallable):
         self.threads = ThreadMaster()
         #self.screencast = False
 
-        self.plugin_manager = PluginManager(
-            self.settings.get_local_plugin_path(),
-            self.settings.get_plugin_settings_path())
         self.effects = EffectsHandler()
-        self.deviceprobe = None
 
         self.projectManager = ProjectManager(self.effects)
         self._connectToProjectManager(self.projectManager)
@@ -160,9 +153,6 @@ class Pitivi(Loggable, Signallable):
             return False
         self.threads.stopAllThreads()
         self.settings.storeSettings()
-        if self.deviceprobe:
-            self.deviceprobe.release()
-        self.deviceprobe = None
         self.current = None
         instance.PiTiVi = None
         self.emit("shutdown")
@@ -279,7 +269,7 @@ class GuiPitivi(InteractivePitivi):
         dialog = gtk.MessageDialog(type=gtk.MESSAGE_ERROR,
                                    buttons=gtk.BUTTONS_OK)
         dialog.set_icon_name("pitivi")
-        dialog.set_markup("<b>"+message+"</b>")
+        dialog.set_markup("<b>" + message + "</b>")
         dialog.format_secondary_text(detail)
         dialog.run()
 
@@ -386,7 +376,6 @@ class StartupWizardGuiPitivi(FullGuiPitivi):
         self.projectManager.connect("new-project-loading", self._hideWizardCb)
         self.projectManager.connect("new-project-failed", self._showWizardCb)
         self.wizard = StartUpWizard(self)
-        self.wizard.hide()
         return FullGuiPitivi._createGui(self)
 
     def _showGui(self):
@@ -427,7 +416,7 @@ class PreviewGuiPitivi(GuiPitivi):
 
     def _newProjectLoaded(self, project):
         # create previewer and set ui
-        previewer = Previewer(self.current, pipeline=None, ui=self.viewer)
+        previewer = Previewer(project, ui=self.viewer)
         self._setActioner(previewer)
         # hack to make the gtk.HScale seek slider UI behave properly
         self.viewer._durationChangedCb(None, project.timeline.duration)
@@ -449,7 +438,7 @@ class RenderingNoGuiPitivi(InteractivePitivi):
 
     def _newProjectLoaded(self, project):
         # create renderer and set output file
-        renderer = Renderer(self.current, outfile=self.outfile)
+        renderer = Renderer(project, outfile=self.outfile)
         self._setActioner(renderer)
 
     def shutdown(self):
