@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # PiTiVi , Non-linear video editor
 #
 #       discoverer.py
@@ -18,22 +17,19 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Discover file multimedia information.
 """
 
 from gettext import gettext as _
-import os.path
+import os
 import gobject
 gobject.threads_init()
 import gst
-import gst.pbutils
-from gst.pbutils import INSTALL_PLUGINS_SUCCESS, \
-        INSTALL_PLUGINS_PARTIAL_SUCCESS, INSTALL_PLUGINS_USER_ABORT, \
-        INSTALL_PLUGINS_STARTED_OK
+from gst import pbutils
 import tempfile
 import hashlib
 
@@ -49,13 +45,13 @@ from pitivi.settings import xdg_cache_home
 # what encoded format it is
 # We will need that in order to create proper Stream objects.
 
+
 class EOSSir(gst.Element):
     __gstdetails__ = (
         "EOSSir",
         "Generic",
         "pushes EOS after the first buffer",
-        "Alessandro Decina <alessandro.d@gmail.com>"
-        )
+        "Alessandro Decina <alessandro.d@gmail.com>")
 
     srctemplate = gst.PadTemplate("src", gst.PAD_SRC,
             gst.PAD_ALWAYS, gst.Caps("ANY"))
@@ -83,6 +79,7 @@ class EOSSir(gst.Element):
         return ret
 gobject.type_register(EOSSir)
 
+
 class Discoverer(Signallable, Loggable):
     """
     Queues requests to discover information about given files.
@@ -101,12 +98,11 @@ class Discoverer(Signallable, Loggable):
     """
 
     __signals__ = {
-        "discovery-error" : ["a", "b", "c" ],
-        "discovery-done" : ["uri", "factory"],
-        "ready" : None,
-        "starting" : None,
-        "missing-plugins": ["uri", "detail", "description"]
-        }
+        "discovery-error": ["a", "b", "c"],
+        "discovery-done": ["uri", "factory"],
+        "ready": None,
+        "starting": None,
+        "missing-plugins": ["uri", "detail", "description"]}
 
     def __init__(self):
         Loggable.__init__(self)
@@ -195,9 +191,9 @@ class Discoverer(Signallable, Loggable):
 
         for message in self.missing_plugin_messages:
             detail = \
-                    gst.pbutils.missing_plugin_message_get_installer_detail(message)
+                    pbutils.missing_plugin_message_get_installer_detail(message)
             description = \
-                    gst.pbutils.missing_plugin_message_get_description(message)
+                    pbutils.missing_plugin_message_get_description(message)
 
             self.missing_plugin_details.append(detail)
             self.missing_plugin_descriptions.append(description)
@@ -207,11 +203,11 @@ class Discoverer(Signallable, Loggable):
     def _installMissingPluginsCallback(self, result, factory):
         rescan = False
 
-        if result in (INSTALL_PLUGINS_SUCCESS,
-                INSTALL_PLUGINS_PARTIAL_SUCCESS):
+        if result in (pbutils.INSTALL_PLUGINS_SUCCESS,
+                pbutils.INSTALL_PLUGINS_PARTIAL_SUCCESS):
             gst.update_registry()
             rescan = True
-        elif result == INSTALL_PLUGINS_USER_ABORT \
+        elif result == pbutils.INSTALL_PLUGINS_USER_ABORT \
                 and factory.getOutputStreams():
             self._emitDone(factory)
         else:
@@ -245,7 +241,7 @@ class Discoverer(Signallable, Loggable):
 
         if not self.current_streams and not missing_plugins:
             # woot, nothing decodable
-            self.error = _('Can not decode file.')
+            self.error = _('Cannot decode file.')
             self.error_detail = _("The given file does not contain audio, "
                     "video or picture streams.")
             self._emitError()
@@ -266,7 +262,7 @@ class Discoverer(Signallable, Loggable):
             # assume it's got infinite duration)
             is_image = have_image and len(self.current_streams) == 1
             if self.current_duration == gst.CLOCK_TIME_NONE and not is_image:
-                self.error =_("Could not establish the duration of the file.")
+                self.error = _("Could not establish the duration of the file.")
                 self.error_detail = _("This clip seems to be in a format "
                         "which cannot be accessed in a random fashion.")
                 self._emitError()
@@ -282,7 +278,7 @@ class Discoverer(Signallable, Loggable):
                 self.missing_plugin_details,
                 self.missing_plugin_descriptions,
                 callback)
-        if res is None or res != INSTALL_PLUGINS_STARTED_OK:
+        if res is None or res != pbutils.INSTALL_PLUGINS_STARTED_OK:
             # no missing-plugins handlers
             if factory.getOutputStreams():
                 self._emitDone(factory)
@@ -291,7 +287,6 @@ class Discoverer(Signallable, Loggable):
 
             return True
 
-
         # plugins are being installed, processing will continue when
         # self._installMissingPluginsCallback is called by the application
         return False
@@ -364,7 +359,7 @@ class Discoverer(Signallable, Loggable):
         if not source:
             self.warning("This is not a media file: %s", self.current_uri)
             self.error = _("No available source handler.")
-            self.error_detail = _("You do not have a GStreamer source element to handle protocol '%s'") % gst.uri_get_protocol(self.current_uri)
+            self.error_detail = _('You do not have a GStreamer source element to handle the "%s" protocol') % gst.uri_get_protocol(self.current_uri)
 
             return None
 
@@ -405,6 +400,20 @@ class Discoverer(Signallable, Loggable):
         self.current_uri = self.queue[0]
         self.info("Analyzing %s", self.current_uri)
 
+        # check if file exists and is readable
+        if gst.uri_get_protocol(self.current_uri) == "file":
+            filename = gst.uri_get_location(self.current_uri)
+            error = None
+            if not os.access(filename, os.F_OK):
+                error = _("File does not exist")
+            elif not os.access(filename, os.R_OK):
+                error = _("File not readable by current user")
+            if error:
+                self.info("Error: %s", self.error)
+                self.error = error
+                self._finishAnalysis("File does not exist or is not readable by the current user")
+                return False
+
         # setup graph and start analyzing
         self.pipeline = gst.Pipeline("Discoverer-%s" % self.current_uri)
 
@@ -468,7 +477,7 @@ class Discoverer(Signallable, Loggable):
             self._finishAnalysis("redirect")
             return
 
-        if gst.pbutils.is_missing_plugin_message(message):
+        if pbutils.is_missing_plugin_message(message):
             self._busMessageMissingPlugins(message)
 
     def _busMessageMissingPlugins(self, message):
@@ -717,7 +726,7 @@ class Discoverer(Signallable, Loggable):
     def _addStreamFromPad(self, pad):
         self._maybeQueryDuration(pad)
         self.debug("adding stream from pad %s caps %s", pad, pad.props.caps)
-        stream  = get_stream_for_pad(pad)
+        stream = get_stream_for_pad(pad)
         self.current_streams.append(stream)
 
         return stream
