@@ -16,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 from gettext import gettext as _
 import gobject
@@ -37,6 +37,7 @@ from pitivi.stream import AudioStream, VideoStream
 from pitivi.timeline.track import Track
 from pitivi.undo import UndoableAction
 
+
 class ProjectSettingsChanged(UndoableAction):
 
     def __init__(self, project, old, new):
@@ -52,6 +53,7 @@ class ProjectSettingsChanged(UndoableAction):
         self.project.setSettings(self.oldsettings)
         self._undone()
 
+
 class ProjectLogObserver(UndoableAction):
 
     def __init__(self, log):
@@ -69,6 +71,7 @@ class ProjectLogObserver(UndoableAction):
         self.log.push(action)
         self.log.commit()
 
+
 class ProjectManager(Signallable, Loggable):
     __signals__ = {
         "new-project-loading": ["uri"],
@@ -80,7 +83,7 @@ class ProjectManager(Signallable, Loggable):
         "closing-project": ["project"],
         "project-closed": ["project"],
         "missing-uri": ["formatter", "uri", "factory"],
-        "reverting-to-saved":["project"],
+        "reverting-to-saved": ["project"],
     }
 
     def __init__(self, avalaible_effects={}):
@@ -156,7 +159,7 @@ class ProjectManager(Signallable, Loggable):
         if self.current is None:
             return True
 
-        if self.emit("closing-project", self.current) == False:
+        if not self.emit("closing-project", self.current):
             return False
 
         self.emit("project-closed", self.current)
