@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # PiTiVi , Non-linear video editor
 #
 #       settings.py
@@ -17,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Settings
@@ -28,12 +27,13 @@ import os
 import platform
 import gst
 from ConfigParser import SafeConfigParser, ParsingError
+import xdg.BaseDirectory as xdg_dirs  # Freedesktop directories spec
 
 from gettext import gettext as _
+from gettext import ngettext
 
 from pitivi.signalinterface import Signallable
-from pitivi.encode import available_muxers, available_video_encoders, \
-     available_audio_encoders, available_combinations, \
+from pitivi.encode import available_combinations, \
      get_compatible_sink_caps
 from pitivi.stream import get_stream_for_caps
 from pitivi.log.loggable import Loggable
@@ -43,6 +43,7 @@ if platform.system() == 'Windows':
 else:
     HOME = 'HOME'
 
+
 def get_bool_env(var):
     value = os.getenv(var)
     if not value:
@@ -55,6 +56,7 @@ def get_bool_env(var):
     else:
         return bool(value)
 
+
 def get_env_by_type(type_, var):
     """
     Returns the environment variable.
@@ -76,49 +78,56 @@ def get_env_by_type(type_, var):
             return type_(os.getenv(var))
         return None
 
+
 def get_env_default(var, default):
     value = os.getenv(var)
     if value:
         return value
     return default
 
+
 def get_dir(path, autocreate=True):
     if autocreate and not os.path.exists(path):
         os.makedirs(path)
     return path
 
+
 def get_dirs(glob):
-    return [d for d in glob.split(os.path.pathsep) if os.path.exists(dir)]
+    return [d for d in glob.split(os.path.pathsep) if os.path.exists(d)]
+
 
 def get_env_dir(var, default, autocreate=True):
     return get_dir(get_env_default(var, default))
 
+
 def get_env_dirs(var, default):
     return get_dirs(get_env_default(var, default))
 
+
 def xdg_config_home(autocreate=True):
-    return get_dir(get_env_default("XDG_CONFIG_HOME",
-        os.path.join(os.getenv(HOME), ".config")), autocreate)
+    return get_dir(xdg_dirs.xdg_config_home, autocreate)
+
 
 def xdg_data_home(autocreate=True):
-    return get_dir(get_env_default("XDG_DATA_HOME",
-        os.path.join(os.getenv(HOME), ".local", "share")), autocreate)
+    return get_dir(xdg_dirs.xdg_data_home, autocreate)
+
 
 def xdg_cache_home(autocreate=True):
-    return get_dir(get_env_default("XDG_CACHE_HOME",
-        os.path.join(os.getenv("HOME"), ".cache")), autocreate)
+    return get_dir(xdg_dirs.xdg_cache_home, autocreate)
+
 
 def xdg_data_dirs():
-    return get_env_dirs("XDG_DATA_DIRS",
-        os.path.pathsep.join((os.path.join("usr", "local", "share"),
-            os.path.join("usr", "share"))))
+    return get_env_dirs(xdg_dirs.xdg_data_dirs)
+
 
 def xdg_config_dirs():
-    return get_env_dirs("XDG_CONFIG_DIRS", "/etc/xdg")
+    return get_env_dirs(xdg_dirs.xdg_config_dirs)
+
 
 class ConfigError(Exception):
     pass
 
+
 class Notification(object):
 
     """A descriptor to help with the implementation of signals"""
@@ -134,6 +143,7 @@ class Notification(object):
         setattr(instance, self.attrname, value)
         instance.emit(self.signame)
 
+
 class GlobalSettings(Signallable):
     """
     Global PiTiVi settings.
@@ -241,35 +251,6 @@ class GlobalSettings(Signallable):
         return get_dir(os.path.join(xdg_config_home(autocreate), "pitivi"),
             autocreate)
 
-    def get_local_plugin_path(self, autocreate=True):
-        """
-        Compute the absolute path to local plugin repository
-
-        @param autocreate: create the path if missing
-        @return: the plugin repository path
-        """
-
-        return get_dir(
-            os.path.join(
-                get_dir(
-                    os.path.join(
-                        xdg_data_home(autocreate),
-                        "pitivi"),
-                    autocreate),
-                "plugins"),
-            autocreate)
-
-    def get_plugin_settings_path(self, autocreate=True):
-        """
-        Compute the absolute path to local plugin settings' repository
-
-        @param autocreate: create the path if missing
-        @return: the plugin settings path
-        """
-
-        return get_dir(os.path.join(self.get_local_settings_path(autocreate),
-            "plugin-settings"), autocreate)
-
     def iterAllOptions(self):
         """
         Iterate over all registered options
@@ -368,6 +349,7 @@ class GlobalSettings(Signallable):
             raise ConfigError("Duplicate Section \"%s\"." % section)
         cls.options[section] = {}
 
+
 class StreamEncodeSettings(object):
     """
     Settings for encoding a L{MultimediaStream}.
@@ -422,6 +404,7 @@ class StreamEncodeSettings(object):
     def __str__(self):
         return "<StreamEncodeSettings %s>" % self.encoder
 
+
 class RenderSettings(object):
     """
     Settings for rendering and multiplexing one or multiple streams.
@@ -442,18 +425,22 @@ class RenderSettings(object):
     def __str__(self):
         return "<RenderSettings %s [%d streams]>" % (self.muxer, len(self.settings))
 
+
 class ExportSettings(Signallable, Loggable):
     """
     Multimedia export settings
 
     Signals:
-
     'settings-changed' : the settings have changed
     'encoders-changed' : The encoders or muxer have changed
+
+    @ivar render_scale: The scale to be applied to the video width and height
+    when rendering.
+    @type render_scale: int
     """
     __signals__ = {
-        "settings-changed" : None,
-        "encoders-changed" : None,
+        "settings-changed": None,
+        "encoders-changed": None,
         }
 
     # Audio/Video settings for processing/export
@@ -461,30 +448,33 @@ class ExportSettings(Signallable, Loggable):
     # TODO : Add PAR/DAR for video
     # TODO : switch to using GstFraction internally where appliable
 
+    muxers, aencoders, vencoders = available_combinations()
+
     def __init__(self, **unused_kw):
         Loggable.__init__(self)
         self.videowidth = 720
         self.videoheight = 576
+        self.render_scale = 100
         self.videorate = gst.Fraction(25, 1)
-        self.videopar = gst.Fraction(1, 1)
+        self.videopar = gst.Fraction(16, 15)
         self.audiochannels = 2
         self.audiorate = 44100
         self.audiodepth = 16
         self.vencoder = "theoraenc"
         self.aencoder = "vorbisenc"
         self.muxer = "oggmux"
-        self.containersettings = {}
-        self.acodecsettings = {}
-        self.vcodecsettings = {}
-        self.muxers = available_muxers()
-        self.vencoders = available_video_encoders()
-        self.aencoders = available_audio_encoders()
-        self.muxers = available_combinations(self.muxers, self.vencoders, self.aencoders)
+        # A (muxer -> containersettings) map.
+        self._containersettings_cache = {}
+        # A (vencoder -> vcodecsettings) map.
+        self._vcodecsettings_cache = {}
+        # A (aencoder -> acodecsettings) map.
+        self._acodecsettings_cache = {}
 
     def copy(self):
         ret = ExportSettings()
         ret.videowidth = self.videowidth
         ret.videoheight = self.videoheight
+        ret.render_scale = self.render_scale
         ret.videorate = gst.Fraction(self.videorate.num, self.videorate.denom)
         ret.videopar = gst.Fraction(self.videopar.num, self.videopar.denom)
         ret.audiochannels = self.audiochannels
@@ -498,59 +488,58 @@ class ExportSettings(Signallable, Loggable):
         ret.vcodecsettings = dict(self.vcodecsettings)
         return ret
 
+    def getDAR(self):
+        return gst.Fraction(self.videowidth, self.videoheight) * self.videopar
+
     def __str__(self):
         msg = _("Export Settings\n")
         msg += _("Video: ") + str(self.videowidth) + " " + str(self.videoheight) +\
-               " " + str(self.videorate) + " " + str (self.videopar)
-        msg += "\n\t" + str(self.vencoder) + " " +str(self.vcodecsettings)
+               " " + str(self.videorate) + " " + str(self.videopar)
+        msg += "\n\t" + str(self.vencoder) + " " + str(self.vcodecsettings)
         msg += _("\nAudio: ") + str(self.audiochannels) + " " + str(self.audiorate) +\
                " " + str(self.audiodepth)
         msg += "\n\t" + str(self.aencoder) + " " + str(self.acodecsettings)
         msg += _("\nMuxer: ") + str(self.muxer) + " " + str(self.containersettings)
         return msg
 
-    def getVideoCaps(self):
+    def getVideoWidthAndHeight(self, render=False):
+        """ Returns the video width and height as a tuple
+
+        @param render: Whether to apply self.render_scale to the returned
+        values.
+        @type render: bool
+        """
+        if render:
+            scale = self.render_scale
+        else:
+            scale = 100
+        return self.videowidth * scale / 100, self.videoheight * scale / 100
+
+    def getVideoCaps(self, render=False):
         """ Returns the GstCaps corresponding to the video settings """
-        astr = "width=%d,height=%d,pixel-aspect-ratio=%d/%d,framerate=%d/%d" % (self.videowidth,
-                                                                                self.videoheight,
-                                                                                self.videopar.num,
-                                                                                self.videopar.denom,
-                                                                                self.videorate.num,
-                                                                                self.videorate.denom)
-        vcaps = gst.caps_from_string("video/x-raw-yuv,%s;video/x-raw-rgb,%s" % (astr, astr))
+        videowidth, videoheight = self.getVideoWidthAndHeight(render=render)
+        vstr = "width=%d,height=%d,pixel-aspect-ratio=%d/%d,framerate=%d/%d" % (
+                videowidth, videoheight,
+                self.videopar.num, self.videopar.denom,
+                self.videorate.num, self.videorate.denom)
+        caps_str = "video/x-raw-yuv,%s;video/x-raw-rgb,%s" % (vstr, vstr)
+        video_caps = gst.caps_from_string(caps_str)
         if self.vencoder:
-            return get_compatible_sink_caps(self.vencoder, vcaps)
-        return vcaps
-
-    def getVideoDescription(self):
-        """ Returns a human-readable markup-ed string describing the video properties """
-        res = "%d x %d <i>pixels</i> at %.2f <i>fps</i>" % (self.videowidth,
-                self.videoheight, float(self.videorate))
-        if self.vencoder is not None:
-            res += " <i>(%s)</i>" % self.vencoder
-
-        return res
-
-    def getAudioDescription(self):
-        """ Returns a human-readable markup-ed string describing the audio properties """
-        res = "%d channels at %d <i>Hz</i> (%d <i>bits</i>)" % \
-                (self.audiochannels, self.audiorate, self.audiodepth)
-        if self.aencoder is not None:
-            res += " <i>(%s)</i>" % self.aencoder
-
-        return res
+            return get_compatible_sink_caps(self.vencoder, video_caps)
+        return video_caps
 
     def getAudioCaps(self):
         """ Returns the GstCaps corresponding to the audio settings """
-        astr = "rate=%d,channels=%d" % (self.audiorate, self.audiochannels)
-        astrcaps = gst.caps_from_string("audio/x-raw-int,%s;audio/x-raw-float,%s" % (astr, astr))
+        astr = "rate=%d,channels=%d,depth=%d" % (
+                self.audiorate, self.audiochannels, self.audiodepth)
+        caps_str = "audio/x-raw-int,%s;audio/x-raw-float,%s" % (astr, astr)
+        audio_caps = gst.caps_from_string(caps_str)
         if self.aencoder:
-            return get_compatible_sink_caps(self.aencoder, astrcaps)
-        return astrcaps
+            return get_compatible_sink_caps(self.aencoder, audio_caps)
+        return audio_caps
 
-        # interset with current audioencoder sink pad caps
-
-    def setVideoProperties(self, width=-1, height=-1, framerate=-1, par=-1):
+    def setVideoProperties(self, width=-1, height=-1, framerate=-1, par=-1,
+            render_scale=-1):
         """ Set the video width, height and framerate """
         self.info("set_video_props %d x %d @ %r fps", width, height, framerate)
         changed = False
@@ -560,6 +549,9 @@ class ExportSettings(Signallable, Loggable):
         if not height == -1 and not height == self.videoheight:
             self.videoheight = height
             changed = True
+        if not render_scale == -1 and not render_scale == self.render_scale:
+            self.render_scale = render_scale
+            changed = True
         if not framerate == -1 and not framerate == self.videorate:
             self.videorate = framerate
             changed = True
@@ -600,11 +592,48 @@ class ExportSettings(Signallable, Loggable):
         if changed:
             self.emit("encoders-changed")
 
+    @property
+    def containersettings(self):
+        return self._containersettings_cache.setdefault(self.muxer, {})
+
+    @containersettings.setter
+    def containersettings(self, value):
+        self._containersettings_cache[self.muxer] = value
+
+    @property
+    def vcodecsettings(self):
+        return self._vcodecsettings_cache.setdefault(self.vencoder, {})
+
+    @vcodecsettings.setter
+    def vcodecsettings(self, value):
+        self._vcodecsettings_cache[self.vencoder] = value
+
+    @property
+    def acodecsettings(self):
+        return self._acodecsettings_cache.setdefault(self.aencoder, {})
+
+    @acodecsettings.setter
+    def acodecsettings(self, value):
+        self._acodecsettings_cache[self.aencoder] = value
+
+    def getAudioEncoders(self):
+        """ Returns the list of audio encoders compatible with the current
+        muxer """
+        return self.aencoders[self.muxer]
+
+    def getVideoEncoders(self):
+        """ Returns the list of video encoders compatible with the current
+        muxer """
+        return self.vencoders[self.muxer]
+
+
 def export_settings_to_render_settings(export,
         have_video=True, have_audio=True):
+    """Convert the specified ExportSettings object to a RenderSettings object.
+    """
     # Get the audio and video caps/encoder/settings
     astream = get_stream_for_caps(export.getAudioCaps())
-    vstream = get_stream_for_caps(export.getVideoCaps())
+    vstream = get_stream_for_caps(export.getVideoCaps(render=True))
 
     encoder_settings = []
     if export.vencoder is not None and have_video:
