@@ -17,8 +17,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 from bisect import bisect_left
 
@@ -43,10 +43,12 @@ SELECT_ADD = 2
 SELECT_BETWEEN = 3
 """Select a range of clips"""
 
+
 class TimelineError(Exception):
     """Base Exception for errors happening in L{Timeline}s or L{TimelineObject}s"""
     pass
 
+
 class TimelineObject(Signallable, Loggable):
     """
     Base class for contents of a C{Timeline}.
@@ -88,7 +90,7 @@ class TimelineObject(Signallable, Loggable):
         'out-point-changed': ['in-point'],
         'media-duration-changed': ['media-duration'],
         'priority-changed': ['priority'],
-        'selected-changed' : ['state'],
+        'selected-changed': ['state'],
         'track-object-added': ["track_object"],
         'track-object-removed': ["track_object"],
     }
@@ -444,6 +446,7 @@ class TimelineObject(Signallable, Loggable):
 
         self.emit("track-object-removed", obj)
 
+
 class Selection(Signallable):
     """
     A collection of L{TimelineObject}.
@@ -456,7 +459,7 @@ class Selection(Signallable):
     """
 
     __signals__ = {
-        "selection-changed" : []
+        "selection-changed": []
     }
 
     def __init__(self):
@@ -558,7 +561,6 @@ class LinkEntry(object):
         self.duration = duration
 
 
-
 class LinkPropertyChangeTracker(PropertyChangeTracker):
     """
     Tracker for private usage by L{Link}
@@ -572,6 +574,7 @@ class LinkPropertyChangeTracker(PropertyChangeTracker):
 
     property_names = ('start', 'duration')
 
+
 class Link(object):
 
     def __init__(self):
@@ -660,6 +663,7 @@ class Link(object):
         else:
             self.waiting_update.remove(timeline_object)
 
+
 # FIXME: This seems overly complicated and (therefore) a potential speed bottleneck.
 # It would be much simpler to just track objects, and specify for each object
 # which property we would like to track (start, end, both). We could then have
@@ -711,7 +715,7 @@ class TimelineEdges(object):
         """
         self._disconnectFromTimelineObject(timeline_object)
         for obj in timeline_object.track_objects:
-             self.removeTrackObject(obj)
+            self.removeTrackObject(obj)
 
     def _connectToTimelineObject(self, timeline_object):
         timeline_object.connect("track-object-added", self._trackObjectAddedCb)
@@ -919,6 +923,7 @@ class TimelineEdges(object):
             return self.by_start[end]
         return []
 
+
 class EditingContext(object):
 
     DEFAULT = 0
@@ -1218,7 +1223,7 @@ class MoveContext(EditingContext):
         if len(self.timeline_objects) == 1:
             if not self._overlapsAreTransitions(focus_timeline_object,
                 priority):
-                self._defaultTo (initial_position, initial_priority)
+                self._defaultTo(initial_position, initial_priority)
             EditingContext.finish(self)
             return
 
@@ -1238,7 +1243,6 @@ class MoveContext(EditingContext):
                 else:
                     priority += 1
 
-
                 self._defaultTo(final_position, priority)
             else:
                 overlap = False
@@ -1269,7 +1273,7 @@ class MoveContext(EditingContext):
         position = max(self.min_position, position)
 
         self.focus.priority = priority
-        self.focus.setStart(position, snap = self._snap)
+        self.focus.setStart(position, snap=self._snap)
 
         for obj, (s_offset, p_offset) in self.offsets.iteritems():
             obj.setStart(position + s_offset)
@@ -1313,6 +1317,7 @@ class MoveContext(EditingContext):
 
         return position, priority
 
+
 class TrimStartContext(EditingContext):
 
     def __init__(self, timeline, focus, other):
@@ -1390,8 +1395,8 @@ class TrimStartContext(EditingContext):
             self._defaultTo(position, self.focus.priority)
         EditingContext.finish(self)
 
-class TrimEndContext(EditingContext):
 
+class TrimEndContext(EditingContext):
     def __init__(self, timeline, focus, other):
         EditingContext.__init__(self, timeline, focus, other)
         self.adjacent = timeline.edges.getObjsAdjacentToEnd(focus)
@@ -1466,6 +1471,7 @@ class TrimEndContext(EditingContext):
             duration = absolute_initial_duration + right_gap.duration
             self._defaultTo(duration, self.focus.priority)
 
+
 class Timeline(Signallable, Loggable):
     """
     Top-level container for L{TimelineObject}s.
@@ -1561,7 +1567,7 @@ class Timeline(Signallable, Loggable):
             self.duration = duration
             self.emit('duration-changed', duration)
 
-    def updateVideoCaps (self, caps= None):
+    def updateVideoCaps(self, caps=None):
         if caps:
             self._video_caps = caps
 
@@ -1636,6 +1642,18 @@ class Timeline(Signallable, Loggable):
         for obj in objs:
             self.removeTimelineObject(obj, deep=True)
 
+    def usesFactory(self, factory):
+        """
+        Return whether the specified factory is present in the timeline.
+        @param the factory you are looking for
+        @type factory
+        @returns True if found, or False if not in the timeline.
+        """
+        for obj in self.timeline_objects:
+            if obj.factory is factory:
+                return True
+        return False
+
     def _timelineObjectStartChangedCb(self, timeline_object, start):
         self.timeline_objects.remove(timeline_object)
         start_insort_right(self.timeline_objects, timeline_object)
@@ -1734,10 +1752,10 @@ class Timeline(Signallable, Loggable):
                 break
 
         if track is None:
-          raise TimelineError("There is no Track to add the effect to")
+            raise TimelineError("There is no Track to add the effect to")
 
         if not timeline_objects:
-          raise TimelineError("There is no timeline object to add effect to")
+            raise TimelineError("There is no timeline object to add effect to")
 
         listTimelineObjectTrackObject = []
         track_object = TrackEffect(factory, input_stream)
@@ -1752,7 +1770,7 @@ class Timeline(Signallable, Loggable):
             listTimelineObjectTrackObject.append((obj, copy_track_obj))
 
         self.debug("%s", ["TimelineObject %s => Track object: %s |"\
-                           %(listTo[0], listTo[1])\
+                           % (listTo[0], listTo[1])\
                            for listTo in listTimelineObjectTrackObject])
         return listTimelineObjectTrackObject
 
@@ -2056,12 +2074,12 @@ class Timeline(Signallable, Loggable):
 
         # Exclude objects that start after current position.
         for obj in self.timeline_objects:
-                tl_objs.append(obj)
-                if obj.start > time:
-                    break
+            tl_objs.append(obj)
+            if obj.start > time:
+                break
 
         keyframe_positions = self._getKeyframePositions(tl_objs)
-        for n in range(len(keyframe_positions) -1, -1, -1):
+        for n in range(len(keyframe_positions) - 1, -1, -1):
             if keyframe_positions[n] < time:
                 return keyframe_positions[n]
         return None
@@ -2079,8 +2097,8 @@ class Timeline(Signallable, Loggable):
 
         keyframe_positions = self._getKeyframePositions(tl_objs)
         for n in range(0, len(keyframe_positions)):
-                if keyframe_positions[n] > time:
-                    return keyframe_positions[n]
+            if keyframe_positions[n] > time:
+                return keyframe_positions[n]
 
         return None
 
