@@ -16,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 A collection of helper classes and routines for dynamically creating user
@@ -29,13 +29,18 @@ import re
 import sys
 import gst
 from gettext import gettext as _
+from pitivi.utils import time_to_string
 from pitivi.ui.common import unpack_color, pack_color_32, pack_color_64
 import pango
+from pitivi.ui.common import PADDING, SPACING
+
 
 class DynamicWidget(object):
 
     """An interface which provides a uniform way to get, set, and observe
     widget properties"""
+    def __init__(self, default):
+        self.default = default
 
     def connectValueChanged(self, callback, *args):
         raise NotImplementedError
@@ -46,12 +51,24 @@ class DynamicWidget(object):
     def getWidgetValue(self, value):
         raise NotImplementedError
 
-class DefaultWidget(gtk.Label):
+    def getWidgetDefault(self):
+        return self.default
+
+    def setWidgetDefault(self, value):
+        self.default = value
+
+    def setWidgetToDefault(self):
+        if self.default is not None:
+            self.setWidgetValue(self.default)
+
+
+class DefaultWidget(gtk.Label, DynamicWidget):
 
     """When all hope fails...."""
 
-    def __init__(self, *unused, **kw_unused):
+    def __init__(self, default=None, *unused, **kw_unused):
         gtk.Label.__init__(self, _("Implement Me"))
+        DynamicWidget.__init__(self, default)
 
     def connectValueChanged(self, callback, *args):
         pass
@@ -63,7 +80,7 @@ class DefaultWidget(gtk.Label):
         return self.get_text()
 
 
-class TextWidget(gtk.HBox):
+class TextWidget(gtk.HBox, DynamicWidget):
 
     """A gtk.Entry which emits a value-changed signal only when its input is
     valid (matches the provided regex). If the input is invalid, a warning
@@ -71,7 +88,7 @@ class TextWidget(gtk.HBox):
 
     __gtype_name__ = 'TextWidget'
     __gsignals__ = {
-        "value-changed" : (
+        "value-changed": (
             gobject.SIGNAL_RUN_LAST,
             gobject.TYPE_NONE,
             (),)
@@ -80,8 +97,10 @@ class TextWidget(gtk.HBox):
     __INVALID__ = gtk.gdk.Color(0xFFFF, 0, 0)
     __NORMAL__ = gtk.gdk.Color(0, 0, 0)
 
-    def __init__(self, matches = None, choices = None):
+    def __init__(self, matches=None, choices=None, default=None):
         gtk.HBox.__init__(self)
+        DynamicWidget.__init__(self, default)
+
         self.set_border_width(0)
         self.set_spacing(0)
         if choices:
@@ -98,6 +117,7 @@ class TextWidget(gtk.HBox):
         self.matches = None
         self.last_valid = None
         self.valid = False
+        self.send_signal = True
         self.text.connect("changed", self._textChanged)
         if matches:
             if type(matches) is str:
@@ -109,7 +129,8 @@ class TextWidget(gtk.HBox):
     def connectValueChanged(self, callback, *args):
         return self.connect("value-changed", callback, *args)
 
-    def setWidgetValue(self, value):
+    def setWidgetValue(self, value, send_signal=True):
+        self.send_signal = send_signal
         self.text.set_text(value)
 
     def getWidgetValue(self):
@@ -117,12 +138,17 @@ class TextWidget(gtk.HBox):
             return self.last_valid
         return self.text.get_text()
 
+    def addChoices(self, choices):
+        for choice in choices:
+            self.combo.append_text(choice)
+
     def _textChanged(self, unused_widget):
         text = self.text.get_text()
         if self.matches:
             if self._filter(text):
                 self.last_valid = text
-                self.emit("value-changed")
+                if self.send_signal:
+                    self.emit("value-changed")
                 if not self.valid:
                     self.text.set_icon_from_stock(1, None)
                 self.valid = True
@@ -130,30 +156,41 @@ class TextWidget(gtk.HBox):
                 if self.valid:
                     self.text.set_icon_from_stock(1, gtk.STOCK_DIALOG_WARNING)
                 self.valid = False
-        else:
+        elif self.send_signal:
             self.emit("value-changed")
 
+        self.send_signal = True
+
     def _filter(self, text):
         match = self.matches.match(text)
         if match is not None:
             return True
         return False
 
-class NumericWidget(gtk.HBox):
+    def set_width_chars(self, width):
+        """Allows setting the width of the text entry widget for compactness."""
+        self.text.set_width_chars(width)
+
+
+class NumericWidget(gtk.HBox, DynamicWidget):
 
     """A gtk.HScale and a gtk.SpinButton which share an adjustment. The
     SpinButton is always displayed, while the HScale only appears if both
     lower and upper bounds are defined"""
 
-    def __init__(self, upper = None, lower = None):
+    def __init__(self, upper=None, lower=None, default=None):
         gtk.HBox.__init__(self)
+        DynamicWidget.__init__(self, default)
 
+        self.spacing = SPACING
         self.adjustment = gtk.Adjustment()
         self.upper = upper
         self.lower = lower
-        if (upper != None) and (lower != None):
+        self._type = None
+        if (upper != None) and (lower != None) and\
+            (upper < 5000) and (lower > -5000):
             self.slider = gtk.HScale(self.adjustment)
-            self.pack_end(self.slider)
+            self.pack_start(self.slider, fill=True, expand=True)
             self.slider.show()
             self.slider.props.draw_value = False
 
@@ -165,35 +202,77 @@ class NumericWidget(gtk.HBox):
         self.adjustment.props.lower = lower
         self.adjustment.props.upper = upper
         self.spinner = gtk.SpinButton(self.adjustment)
-        self.pack_start(self.spinner)
+        self.pack_end(self.spinner, expand=not hasattr(self, 'slider'))
         self.spinner.show()
 
     def connectValueChanged(self, callback, *args):
         self.adjustment.connect("value-changed", callback, *args)
 
     def getWidgetValue(self):
+        if self._type:
+            return self._type(self.adjustment.get_value())
+
         return self.adjustment.get_value()
 
     def setWidgetValue(self, value):
         type_ = type(value)
-        if type_ == int:
+        if self._type is None:
+            self._type = type_
+
+        if type_ == int or type_ == long:
             minimum, maximum = (-sys.maxint, sys.maxint)
             step = 1.0
             page = 10.0
         elif type_ == float:
             minimum, maximum = (gobject.G_MINDOUBLE, gobject.G_MAXDOUBLE)
-            step = 0.00001
-            page = 0.01
+            step = 0.01
+            page = 0.1
             self.spinner.props.digits = 2
         if self.lower is not None:
             minimum = self.lower
         if self.upper is not None:
             maximum = self.upper
         self.adjustment.set_all(value, minimum, maximum, step, page, 0)
-        self.spinner.props.climb_rate = 0.01 * abs(min(maximum, 1000) -
-            max(minimum, -1000))
+        self.spinner.set_adjustment(self.adjustment)
 
-class FractionWidget(TextWidget):
+
+class TimeWidget(TextWidget, DynamicWidget):
+    """ A widget that contains a time in nanosconds"""
+
+    regex = re.compile("^([0-9]:[0-5][0-9]:[0-5][0-9])\.[0-9][0-9][0-9]$")
+    __gtype_name__ = 'TimeWidget'
+
+    def __init__(self, default=None):
+        DynamicWidget.__init__(self, default)
+        TextWidget.__init__(self, self.regex)
+        TextWidget.set_width_chars(self, 10)
+
+    def getWidgetValue(self):
+        timecode = TextWidget.getWidgetValue(self)
+
+        hh, mm, end = timecode.split(":")
+        ss, xxx = end.split(".")
+        nanosecs = int(hh) * 3.6 * 10e12 \
+            + int(mm) * 6 * 10e10 \
+            + int(ss) * 10e9 \
+            + int(xxx) * 10e6
+
+        nanosecs = nanosecs / 10  # Compensate the 10 factor of e notation
+
+        return nanosecs
+
+    def setWidgetValue(self, value, send_signal=True):
+        TextWidget.setWidgetValue(self, time_to_string(value),
+                                send_signal=send_signal)
+
+    def connectFocusEvents(self, focusInCb, focusOutCb):
+        fIn = self.text.connect("button-press-event", focusInCb)
+        fOut = self.text.connect("focus-out-event", focusOutCb)
+
+        return [fIn, fOut]
+
+
+class FractionWidget(TextWidget, DynamicWidget):
 
     """A gtk.ComboBoxEntry """
 
@@ -201,7 +280,9 @@ class FractionWidget(TextWidget):
         "^([0-9]*(\.[0-9]+)?)(([:/][0-9]*(\.[0-9]+)?)|M)?$")
     __gtype_name__ = 'FractionWidget'
 
-    def __init__(self, range=None, presets=None):
+    def __init__(self, range=None, presets=None, default=None):
+        DynamicWidget.__init__(self, default)
+
         if range:
             flow = float(range.low)
             fhigh = float(range.high)
@@ -230,6 +311,20 @@ class FractionWidget(TextWidget):
                 return True
         return False
 
+    def addPresets(self, presets):
+        choices = []
+        for preset in presets:
+            if type(preset) is str:
+                strval = preset
+                preset = self._parseText(preset)
+            else:
+                strval = "%g:%g" % (preset.num, preset.denom)
+            fpreset = float(preset)
+            if self.low <= fpreset and fpreset <= self.high:
+                choices.append(strval)
+
+        self.addChoices(choices)
+
     def setWidgetValue(self, value):
         if type(value) is str:
             value = self._parseText(value)
@@ -262,12 +357,14 @@ class FractionWidget(TextWidget):
                 denom = float(groups[2][1:])
         return gst.Fraction(num, denom)
 
-class ToggleWidget(gtk.CheckButton):
+
+class ToggleWidget(gtk.CheckButton, DynamicWidget):
 
     """A gtk.CheckButton which supports the DynamicWidget interface."""
 
-    def __init__(self):
+    def __init__(self, default=None):
         gtk.CheckButton.__init__(self)
+        DynamicWidget.__init__(self, default)
 
     def connectValueChanged(self, callback, *args):
         self.connect("toggled", callback, *args)
@@ -278,14 +375,16 @@ class ToggleWidget(gtk.CheckButton):
     def getWidgetValue(self):
         return self.get_active()
 
-class ChoiceWidget(gtk.HBox):
+
+class ChoiceWidget(gtk.HBox, DynamicWidget):
 
     """Abstractly, represents a choice between a list of named values. The
     association between value names and values is arbitrary. The current
     implementation uses a gtk.ComboBox."""
 
-    def __init__(self, choices):
+    def __init__(self, choices, default=None):
         gtk.HBox.__init__(self)
+        DynamicWidget.__init__(self, default)
         self.choices = None
         self.values = None
         self.contents = gtk.combo_box_new_text()
@@ -319,7 +418,8 @@ class ChoiceWidget(gtk.HBox):
         else:
             self.contents.set_sensitive(True)
 
-class PresetChoiceWidget(gtk.HBox):
+
+class PresetChoiceWidget(gtk.HBox, DynamicWidget):
 
     """A popup which manages preset settings on a group of widgets supporting
     the dynamic interface"""
@@ -357,8 +457,9 @@ class PresetChoiceWidget(gtk.HBox):
         def unmap(self):
             return [w.getWidgetValue() for w in self.widgets if w]
 
-    def __init__(self, presets):
+    def __init__(self, presets, default=None):
         gtk.HBox.__init__(self)
+        DynamicWidget.__init__(self, default)
         self._block_update = False
         self._widget_map = None
 
@@ -444,23 +545,24 @@ class PresetChoiceWidget(gtk.HBox):
         self.combo.set_active(preset_index)
 
 
-class PathWidget(gtk.FileChooserButton):
+class PathWidget(gtk.FileChooserButton, DynamicWidget):
 
     """A gtk.FileChooserButton which supports the DynamicWidget interface."""
 
     __gtype_name__ = 'PathWidget'
 
     __gsignals__ = {
-        "value-changed" : (gobject.SIGNAL_RUN_LAST,
+        "value-changed": (gobject.SIGNAL_RUN_LAST,
             gobject.TYPE_NONE,
             ()),
     }
 
-    def __init__(self, action = gtk.FILE_CHOOSER_ACTION_OPEN):
+    def __init__(self, action=gtk.FILE_CHOOSER_ACTION_OPEN, default=None):
+        DynamicWidget.__init__(self, default)
         self.dialog = gtk.FileChooserDialog(
-            action = action,
-            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_CLOSE,
-            gtk.RESPONSE_CLOSE))
+            action=action,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_CLOSE,
+             gtk.RESPONSE_CLOSE))
         self.dialog.set_default_response(gtk.RESPONSE_OK)
         gtk.FileChooserButton.__init__(self, self.dialog)
         self.set_title(_("Choose..."))
@@ -483,10 +585,12 @@ class PathWidget(gtk.FileChooserButton):
             self.emit("value-changed")
             self.dialog.hide()
 
-class ColorWidget(gtk.ColorButton):
 
-    def __init__(self, value_type=str):
+class ColorWidget(gtk.ColorButton, DynamicWidget):
+
+    def __init__(self, value_type=str, default=None):
         gtk.ColorButton.__init__(self)
+        DynamicWidget.__init__(self, default)
         self.value_type = value_type
         self.set_use_alpha(True)
 
@@ -521,10 +625,12 @@ class ColorWidget(gtk.ColorButton):
             return color
         return color.to_string()
 
-class FontWidget(gtk.FontButton):
 
-    def __init__(self):
+class FontWidget(gtk.FontButton, DynamicWidget):
+
+    def __init__(self, default=None):
         gtk.FontButton.__init__(self)
+        DynamicWidget.__init__(self, default)
         self.set_use_font(True)
 
     def connectValueChanged(self, callback, *args):
@@ -536,10 +642,12 @@ class FontWidget(gtk.FontButton):
     def getWidgetValue(self):
         return self.get_font_name()
 
-class ResolutionWidget(gtk.HBox):
 
-    def __init__ (self):
+class ResolutionWidget(gtk.HBox, DynamicWidget):
+
+    def __init__(self, default=None):
         gtk.HBox.__init__(self)
+        DynamicWidget.__init__(self, default)
         self.props.spacing = 6
 
         self.dwidth = 0
@@ -586,20 +694,16 @@ if __name__ == '__main__':
             ("pear", "pear")),)),
         (ColorWidget, 0x336699FF, (int,)),
         (FontWidget, "Sans 9", ()),
-        (FractionWidget, "30M", 
-            (gst.FractionRange(gst.Fraction(1, 1), 
+        (FractionWidget, "30M",
+            (gst.FractionRange(gst.Fraction(1, 1),
                 gst.Fraction(30000, 1001)),)),
-        (FractionWidget, gst.Fraction(25000, 1001), 
+        (FractionWidget, gst.Fraction(25000, 1001),
             (
                 gst.FractionRange(
-                    gst.Fraction(1, 1), 
+                    gst.Fraction(1, 1),
                     gst.Fraction(30000, 1001)
                 ),
-                (
-                    "25:1", 
-                    gst.Fraction(30,1),
-                    "30M",
-                ),
+                ("25:1", gst.Fraction(30, 1), "30M", ),
             )
         ),
     )
