@@ -16,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 import gtk
 import pango
@@ -57,6 +57,7 @@ GlobalSettings.addConfigOption('lastEffectView',
 INVISIBLE = gtk.gdk.pixbuf_new_from_file(os.path.join(get_pixmap_dir(),
     "invisible.png"))
 
+
 class EffectList(gtk.VBox, Loggable):
     """ Widget for listing effects """
 
@@ -77,7 +78,6 @@ class EffectList(gtk.VBox, Loggable):
         self._current_effect_name = None
         self._current_tooltip_icon = None
 
-
         #Searchbox and combobox
         hfilters = gtk.HBox()
         hfilters.set_spacing(SPACING)
@@ -88,7 +88,6 @@ class EffectList(gtk.VBox, Loggable):
         self.effectCategory = gtk.combo_box_new_text()
         self.effectType.set_active(VIDEO_EFFECT)
 
-
         hfilters.pack_start(self.effectType, expand=True)
         hfilters.pack_end(self.effectCategory, expand=True)
 
@@ -155,28 +154,28 @@ class EffectList(gtk.VBox, Loggable):
         self.iconview_scrollwin.add(self.iconview)
         self.iconview.set_property("has_tooltip", True)
 
-        self.effectType.connect ("changed", self._effectTypeChangedCb)
+        self.effectType.connect("changed", self._effectTypeChangedCb)
 
-        self.effectCategory.connect ("changed", self._effectCategoryChangedCb)
+        self.effectCategory.connect("changed", self._effectCategoryChangedCb)
 
-        self.searchEntry.connect ("changed", self.searchEntryChangedCb)
-        self.searchEntry.connect ("button-press-event", self.searchEntryActivateCb)
-        self.searchEntry.connect ("focus-out-event", self.searchEntryDesactvateCb)
-        self.searchEntry.connect ("icon-press", self.searchEntryIconClickedCb)
+        self.searchEntry.connect("changed", self.searchEntryChangedCb)
+        self.searchEntry.connect("button-press-event", self.searchEntryActivateCb)
+        self.searchEntry.connect("focus-out-event", self.searchEntryDesactvateCb)
+        self.searchEntry.connect("icon-press", self.searchEntryIconClickedCb)
 
         self.treeview.connect("button-press-event", self._buttonPressEventCb)
         self.treeview.connect("select-cursor-row", self._enterPressEventCb)
         self.treeview.connect("motion-notify-event", self._motionNotifyEventCb)
         self.treeview.connect("query-tooltip", self._queryTooltipCb)
         self.treeview.connect("button-release-event", self._buttonReleaseCb)
-        self.treeview.drag_source_set(0,[], gtk.gdk.ACTION_COPY)
+        self.treeview.drag_source_set(0, [], gtk.gdk.ACTION_COPY)
         self.treeview.connect("drag_begin", self._dndDragBeginCb)
         self.treeview.connect("drag_data_get", self._dndDataGetCb)
 
         self.iconview.connect("button-press-event", self._buttonPressEventCb)
         self.iconview.connect("activate-cursor-item", self._enterPressEventCb)
         self.iconview.connect("query-tooltip", self._queryTooltipCb)
-        self.iconview.drag_source_set(0,[], gtk.gdk.ACTION_COPY)
+        self.iconview.drag_source_set(0, [], gtk.gdk.ACTION_COPY)
         self.iconview.connect("motion-notify-event", self._motionNotifyEventCb)
         self.iconview.connect("button-release-event", self._buttonReleaseCb)
         self.iconview.connect("drag_begin", self._dndDragBeginCb)
@@ -204,36 +203,36 @@ class EffectList(gtk.VBox, Loggable):
         hsearch.show_all()
 
     def _addMenuItems(self, uiman):
-      view_menu_item = uiman.get_widget('/MainMenuBar/View')
-      view_menu = view_menu_item.get_submenu()
-      seperator = gtk.SeparatorMenuItem()
-      self.treeview_menuitem = gtk.RadioMenuItem(None,
-              _("Show Video Effects as a List"))
-      self.iconview_menuitem = gtk.RadioMenuItem(self.treeview_menuitem,
-              _("Show Video Effects as Icons"))
+        view_menu_item = uiman.get_widget('/MainMenuBar/View')
+        view_menu = view_menu_item.get_submenu()
+        seperator = gtk.SeparatorMenuItem()
+        self.treeview_menuitem = gtk.RadioMenuItem(None,
+                _("Show Video Effects as a List"))
+        self.iconview_menuitem = gtk.RadioMenuItem(self.treeview_menuitem,
+                _("Show Video Effects as Icons"))
 
-      if self.settings.lastEffectView == SHOW_TREEVIEW:
-          self.treeview_menuitem.set_active(True)
-          self.iconview_menuitem.set_active(False)
-      else:
-          self.treeview_menuitem.set_active(False)
-          self.iconview_menuitem.set_active(True)
+        if self.settings.lastEffectView == SHOW_TREEVIEW:
+            self.treeview_menuitem.set_active(True)
+            self.iconview_menuitem.set_active(False)
+        else:
+            self.treeview_menuitem.set_active(False)
+            self.iconview_menuitem.set_active(True)
 
-      self.treeview_menuitem.connect("toggled", self._treeViewMenuItemToggledCb)
-      view_menu.append(seperator)
-      view_menu.append(self.treeview_menuitem)
-      view_menu.append(self.iconview_menuitem)
-      self.treeview_menuitem.show()
-      self.iconview_menuitem.show()
-      seperator.show()
+        self.treeview_menuitem.connect("toggled", self._treeViewMenuItemToggledCb)
+        view_menu.append(seperator)
+        view_menu.append(self.treeview_menuitem)
+        view_menu.append(self.iconview_menuitem)
+        self.treeview_menuitem.show()
+        self.iconview_menuitem.show()
+        seperator.show()
 
-      self.effect_view = self.settings.lastEffectView
+        self.effect_view = self.settings.lastEffectView
 
     def _addFactories(self, elements, effectType):
         for element in elements:
-            name =element.get_name()
+            name = element.get_name()
             effect = self.app.effects.getFactoryFromName(name)
-            self.storemodel.append([ effect.getHumanName(),
+            self.storemodel.append([effect.getHumanName(),
                                      effect.getDescription(), effectType,
                                      effect.getCategories(),
                                      effect, name,
@@ -297,7 +296,7 @@ class EffectList(gtk.VBox, Loggable):
                 return view.path_is_selected(path) and len(selection)
         return False
 
-    def _enterPressEventCb(self, view, event = None):
+    def _enterPressEventCb(self, view, event=None):
         factory_name = self.getSelectedItems()
         self.app.gui.clipconfig.effect_expander.addEffectToCurrentSelection(factory_name)
 
@@ -363,10 +362,10 @@ class EffectList(gtk.VBox, Loggable):
             self._dragStarted = True
 
         if self.effect_view is SHOW_TREEVIEW:
-          if chain_up:
-              gtk.TreeView.do_button_press_event(view, event)
-          else:
-              view.grab_focus()
+            if chain_up:
+                gtk.TreeView.do_button_press_event(view, event)
+            else:
+                view.grab_focus()
 
         return False
 
@@ -378,10 +377,10 @@ class EffectList(gtk.VBox, Loggable):
 
         if self.effect_view is SHOW_TREEVIEW or\
                     self._effect_type_ref == AUDIO_EFFECT:
-            view.set_tooltip_row (tooltip, context[1][0])
+            view.set_tooltip_row(tooltip, context[1][0])
         elif self.effect_view is SHOW_ICONVIEW and\
                      self._effect_type_ref == VIDEO_EFFECT:
-            view.set_tooltip_item (tooltip, context[1][0])
+            view.set_tooltip_item(tooltip, context[1][0])
         name = self.modelFilter.get_value(context[2], COL_ELEMENT_NAME)
         if self._current_effect_name != name:
             self._current_effect_name = name
@@ -395,7 +394,7 @@ class EffectList(gtk.VBox, Loggable):
         txt = "<b>%s:</b>\n%s" % (longname, description)
         if self.effect_view == SHOW_ICONVIEW:
             tooltip.set_icon(None)
-        else :
+        else:
             tooltip.set_icon(self._current_tooltip_icon)
         tooltip.set_markup(txt)
 
@@ -434,10 +433,10 @@ class EffectList(gtk.VBox, Loggable):
     def _effectCategoryChangedCb(self, combobox):
         self.modelFilter.refilter()
 
-    def searchEntryChangedCb (self, entry):
+    def searchEntryChangedCb(self, entry):
         self.modelFilter.refilter()
 
-    def searchEntryIconClickedCb (self, entry, unused, unsed1):
+    def searchEntryIconClickedCb(self, entry, unused, unsed1):
         entry.set_text("")
 
     def searchEntryDesactvateCb(self, entry, event):
@@ -463,7 +462,7 @@ class EffectList(gtk.VBox, Loggable):
         return not bool(view.get_path_at_pos(int(event.x), int(event.y)))
 
     def _getDndTuple(self):
-         if self.effectType.get_active() == VIDEO_EFFECT:
+        if self.effectType.get_active() == VIDEO_EFFECT:
             return [dnd.VIDEO_EFFECT_TUPLE, dnd.EFFECT_TUPLE]
-         else:
+        else:
             return [dnd.AUDIO_EFFECT_TUPLE, dnd.EFFECT_TUPLE]
