@@ -17,8 +17,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Main GTK+ window
@@ -49,7 +49,7 @@ from pitivi.ui.timeline import Timeline
 from pitivi.ui.basetabs import BaseTabs
 from pitivi.ui.viewer import PitiviViewer
 from pitivi.configure import pitivi_version, APPNAME, APPURL, APPMANUALURL, \
-     get_pixmap_dir, LIBDIR
+     get_pixmap_dir, LIBDIR, get_ui_dir
 from pitivi.ui import dnd
 from pitivi.pipeline import Pipeline
 from pitivi.action import ViewAction
@@ -69,7 +69,7 @@ if HAVE_GCONF:
     D_G_INTERFACE = "/desktop/gnome/interface"
 
     for gconf_dir in (D_G_INTERFACE, ):
-        gconf.client_get_default ().add_dir (gconf_dir, gconf.CLIENT_PRELOAD_NONE)
+        gconf.client_get_default().add_dir(gconf_dir, gconf.CLIENT_PRELOAD_NONE)
 
 GlobalSettings.addConfigOption("fileSupportEnabled",
     environment="PITIVI_FILE_SUPPORT",
@@ -122,20 +122,22 @@ GlobalSettings.addConfigOption('lastExportFolder',
 GlobalSettings.addConfigOption('elementSettingsDialogWidth',
     section='export',
     key='element-settings-dialog-width',
-    default = 620)
+    default=620)
 GlobalSettings.addConfigOption('elementSettingsDialogHeight',
     section='export',
     key='element-settings-dialog-height',
-    default = 460)
+    default=460)
 GlobalSettings.addConfigSection("effect-configuration")
 GlobalSettings.addConfigOption('effectVPanedPosition',
     section='effect-configuration',
     key='effect-vpaned-position',
     type_=int)
 
+
 def supported(info):
     return formatter.can_handle_location(info[1])
 
+
 def create_stock_icons():
     """ Creates the pitivi-only stock icons """
     gtk.stock_add([
@@ -150,13 +152,13 @@ def create_stock_icons():
             ('pitivi-group', _('Group'), 0, 0, 'pitivi'),
             ])
     pixmaps = {
-        "pitivi-render" : "pitivi-render-24.png",
-        "pitivi-split" : "pitivi-split-24.svg",
-        "pitivi-keyframe" : "pitivi-keyframe-24.svg",
-        "pitivi-unlink" : "pitivi-unlink-24.svg",
-        "pitivi-link" : "pitivi-relink-24.svg",
-        "pitivi-ungroup" : "pitivi-ungroup-24.svg",
-        "pitivi-group" : "pitivi-group-24.svg",
+        "pitivi-render": "pitivi-render-24.png",
+        "pitivi-split": "pitivi-split-24.svg",
+        "pitivi-keyframe": "pitivi-keyframe-24.svg",
+        "pitivi-unlink": "pitivi-unlink-24.svg",
+        "pitivi-link": "pitivi-relink-24.svg",
+        "pitivi-ungroup": "pitivi-ungroup-24.svg",
+        "pitivi-group": "pitivi-group-24.svg",
     }
     factory = gtk.IconFactory()
     pmdir = get_pixmap_dir()
@@ -176,8 +178,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
     @cvar project: The current project
     @type project: L{Project}
     """
-
-
     def __init__(self, instance):
         """ initialize with the Pitivi object """
         gtk.Window.__init__(self)
@@ -223,17 +223,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.app.action_log.connect("redo", self._actionLogRedo)
         self.app.action_log.connect("cleaned", self._actionLogCleaned)
 
-        # if no webcams available, hide the webcam action
-        if self.app.deviceprobe is not None:
-            # On Windows disable device probe
-            if platform.system() != 'Windows':
-                self.app.deviceprobe.connect("device-added", self._deviceChangeCb)
-                self.app.deviceprobe.connect("device-removed", self._deviceChangeCb)
-                if len(self.app.deviceprobe.getVideoSourceDevices()) < 1:
-                    self.webcam_button.set_sensitive(False)
-        else:
-            self.webcam_button.set_sensitive(False)
-
     def showEncodingDialog(self, project, pause=True):
         """
         Shows the L{EncodingDialog} for the given project Timeline.
@@ -247,10 +236,10 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
         if pause:
             project.pipeline.pause()
-        win = EncodingDialog(self, project)
-        win.window.connect("destroy", self._encodingDialogDestroyCb)
+        dialog = EncodingDialog(self, project)
+        dialog.window.connect("destroy", self._encodingDialogDestroyCb)
         self.set_sensitive(False)
-        win.show()
+        dialog.window.show()
 
     def _encodingDialogDestroyCb(self, unused_dialog):
         self.set_sensitive(True)
@@ -277,37 +266,25 @@ class PitiviMainWindow(gtk.Window, Loggable):
              None, _("Reload the current project"), self._revertToSavedProjectCb),
             ("ProjectSettings", gtk.STOCK_PROPERTIES, _("Project Settings"),
              None, _("Edit the project settings"), self._projectSettingsCb),
-            ("RenderProject", 'pitivi-render' , _("_Render project"),
-             None, _("Render project..."), self._recordCb),
+            ("RenderProject", 'pitivi-render', _("_Render Project..."),
+             None, _("Export your project as a finished movie"), self._recordCb),
             ("Undo", gtk.STOCK_UNDO,
              _("_Undo"),
              "<Ctrl>Z", _("Undo the last operation"), self._undoCb),
             ("Redo", gtk.STOCK_REDO,
              _("_Redo"),
              "<Ctrl>Y", _("Redo the last operation that was undone"), self._redoCb),
-            ("PluginManager", gtk.STOCK_PREFERENCES ,
-             _("_Plugins..."),
-             None, _("Manage plugins"), self._pluginManagerCb),
             ("Preferences", gtk.STOCK_PREFERENCES, _("_Preferences"),
               None, None, self._prefsCb),
-            ("ImportfromCam", gtk.STOCK_ADD ,
-             _("Import from _Webcam..."),
-             None, _("Import Camera stream"), self._ImportWebcam),
-            ("Screencast", gtk.STOCK_ADD ,
-             _("_Make screencast..."),
-             None, _("Capture the desktop"), self._Screencast),
-            ("NetstreamCapture", gtk.STOCK_ADD ,
-             _("_Capture Network Stream..."),
-             None, _("Capture Network Stream"), self._ImportNetstream),
             ("Quit", gtk.STOCK_QUIT, None, None, None, self._quitCb),
             ("About", gtk.STOCK_ABOUT, None, None,
              _("Information about %s") % APPNAME, self._aboutCb),
-            ("UserManual", gtk.STOCK_HELP, _("User manual"),
+            ("UserManual", gtk.STOCK_HELP, _("User Manual"),
              None, None, self._userManualCb),
-            ("File", None, _("_File")),
+            ("File", None, _("_Project")),
             ("Edit", None, _("_Edit")),
             ("View", None, _("_View")),
-            ("Library", None, _("_Project")),
+            ("Library", None, _("_Library")),
             ("Timeline", None, _("_Timeline")),
             ("Viewer", None, _("Previe_w")),
             ("PlayPause", gtk.STOCK_MEDIA_PLAY, None, "space", PLAY,
@@ -350,9 +327,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 # this will be set sensitive when the timeline duration changes
                 action.set_sensitive(False)
                 action.props.is_important = True
-            elif action_name == "ImportfromCam":
-                self.webcam_button = action
-                action.set_sensitive(False)
             elif action_name == "Screencast":
                 # FIXME : re-enable this action once istanbul integration is complete
                 # and upstream istanbul has applied packages for proper interaction.
@@ -361,7 +335,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
             elif action_name in [
                 "ProjectSettings", "Quit", "File", "Edit", "Help", "About",
                 "View", "FullScreen", "FullScreenAlternate", "UserManual",
-                "ImportSourcesFolder", "PluginManager", "PlayPause",
+                "ImportSourcesFolder", "PlayPause",
                 "Project", "FrameForward", "FrameBackward",
                 "ShowHideMainToolbar", "ShowHideTimelineToolbar", "Library",
                 "Timeline", "Viewer", "FrameForward", "FrameBackward",
@@ -384,12 +358,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.uimanager = gtk.UIManager()
         self.add_accel_group(self.uimanager.get_accel_group())
         self.uimanager.insert_action_group(self.actiongroup, 0)
-        if 'pitivi.exe' in __file__.lower():
-            xml = LIBDIR + '\\pitivi.exe'
-        else:
-            xml = __file__
-        self.uimanager.add_ui_from_file(os.path.join(os.path.dirname(
-            os.path.abspath(xml)), "mainwindow.xml"))
+        self.uimanager.add_ui_from_file(os.path.join(get_ui_dir(), "mainwindow.xml"))
 
     def _createUi(self, instance):
         """ Create the graphical interface """
@@ -455,7 +424,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
                                         gtk.Label(_("Effects configurations")))
         self.clipconfig.show()
 
-        self.secondhpaned.pack2(self.propertiestabs, resize= True, shrink=False)
+        self.secondhpaned.pack2(self.propertiestabs, resize=True, shrink=False)
         self.propertiestabs.show()
 
         # Viewer
@@ -589,7 +558,6 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.settings.mainWindowShowMainToolbar = mtb.props.active
         self.settings.mainWindowShowTimelineToolbar = ttb.props.active
 
-
     def _sourceListPlayCb(self, sourcelist, factory):
         self._viewFactory(factory)
 
@@ -622,7 +590,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
 
     def showProjectSettingsDialog(self):
         from projectsettings import ProjectSettingsDialog
-        ProjectSettingsDialog(self, self.app.current).show()
+        ProjectSettingsDialog(self, self.app.current).window.show()
 
     def _quitCb(self, unused_action):
         self._saveWindowSettings()
@@ -643,7 +611,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
             action.props.active
 
     def _userManualCb(self, unused_action):
-        webbrowser.open_new (APPMANUALURL)
+        webbrowser.open_new(APPMANUALURL)
 
     def _aboutResponseCb(self, dialog, unused_response):
         dialog.destroy()
@@ -654,7 +622,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def _aboutCb(self, unused_action):
         abt = gtk.AboutDialog()
         abt.set_name(APPNAME)
-        abt.set_version("v%s" % pitivi_version)
+        abt.set_version(pitivi_version)
         gtk.about_dialog_set_url_hook(self._showWebsiteCb)
         abt.set_website(APPURL)
         authors = ["Edward Hervey <bilboed@bilboed.com>",
@@ -675,6 +643,9 @@ class PitiviMainWindow(gtk.Window, Loggable):
                    "Sarath Lakshman <sarathlakshman@slynux.org>",
                    "Alex Balut <alexandru.balut@gmail.com>"]
         abt.set_authors(authors)
+        translators = _("translator-credits")
+        if translators != "translator-credits":
+            abt.set_translator_credits(translators)
         abt.set_license(_("GNU Lesser General Public License\n"
                           "See http://www.gnu.org/copyleft/lesser.html for more details"))
         abt.set_icon_name("pitivi")
@@ -719,44 +690,12 @@ class PitiviMainWindow(gtk.Window, Loggable):
     def _redoCb(self, action):
         self.app.action_log.redo()
 
-    def _pluginManagerCb(self, unused_action):
-        from pluginmanagerdialog import PluginManagerDialog
-        PluginManagerDialog(self.app.plugin_manager)
-
-    # Import from Webcam callback
-    def _ImportWebcam(self,unused_action):
-        from webcam_managerdialog import WebcamManagerDialog
-        w = WebcamManagerDialog(self.app)
-        w.show()
-
-    # Capture network stream callback
-    def _ImportNetstream(self,unused_action):
-        from netstream_managerdialog import NetstreamManagerDialog
-        NetstreamManagerDialog()
-
-    # screencast callback
-    def _Screencast(self,unused_action):
-        from screencast_managerdialog import ScreencastManagerDialog
-        ScreencastManagerDialog(self.app)
-
-    ## Devices changed
-    def _deviceChangeCb(self, probe, unused_device):
-        if len(probe.getVideoSourceDevices()) < 1:
-            self.webcam_button.set_sensitive(False)
-        else:
-            self.webcam_button.set_sensitive(True)
-
-    def _hideChildWindow(self, window, event):
-        window.hide()
-        return True
-
     def _prefsCb(self, unused_action):
         if not self.prefsdialog:
             from pitivi.ui.prefs import PreferencesDialog
             self.prefsdialog = PreferencesDialog(self.app)
-            self.prefsdialog.set_transient_for(self)
-            self.prefsdialog.connect("delete-event", self._hideChildWindow)
-        self.prefsdialog.show()
+            self.prefsdialog.dialog.set_transient_for(self)
+        self.prefsdialog.run()
 
     def rewind(self, unused_action):
         pass
@@ -809,7 +748,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         Zoomable.setZoomLevel(nearest_zoom_level)
 
     def _projectManagerNewProjectLoadingCb(self, projectManager, uri):
-        if uri != None :
+        if uri != None:
             self.manager.add_item(uri)
         self.log("A NEW project is being loaded, deactivate UI")
 
@@ -842,6 +781,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dialog.set_resizable(False)
         dialog.set_has_separator(False)
         dialog.set_default_response(gtk.RESPONSE_YES)
+        dialog.set_transient_for(self)
 
         primary = gtk.Label()
         primary.set_line_wrap(True)
@@ -859,14 +799,14 @@ class PitiviMainWindow(gtk.Window, Loggable):
                 "changes will be lost")
 
         # put the text in a vbox
-        vbox = gtk.VBox(False, SPACING*2)
+        vbox = gtk.VBox(False, SPACING * 2)
         vbox.pack_start(primary, expand=True, fill=True)
         vbox.pack_start(secondary, expand=True, fill=True)
 
         # make the [[image] text] hbox
         image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
                gtk.ICON_SIZE_DIALOG)
-        hbox = gtk.HBox(False, SPACING*2)
+        hbox = gtk.HBox(False, SPACING * 2)
         hbox.pack_start(image, expand=False)
         hbox.pack_start(vbox, expand=True, fill=True)
         action_area = dialog.get_action_area()
@@ -875,7 +815,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         # stuff the hbox in the dialog
         content_area = dialog.get_content_area()
         content_area.pack_start(hbox, expand=True, fill=True)
-        content_area.set_spacing(SPACING*2)
+        content_area.set_spacing(SPACING * 2)
         hbox.show_all()
 
         response = dialog.run()
@@ -917,13 +857,13 @@ class PitiviMainWindow(gtk.Window, Loggable):
                                             _("All unsaved changes will be lost.")
                                         )
             dialog.set_default_response(gtk.RESPONSE_NO)
+            dialog.set_transient_for(self)
             response = dialog.run()
             dialog.destroy()
-            if response <> gtk.RESPONSE_YES:
+            if response != gtk.RESPONSE_YES:
                 return False
         return True
 
-
     def _projectManagerNewProjectFailedCb(self, projectManager, uri, exception):
         project_filename = unquote(uri.split("/")[-1])
         dialog = gtk.MessageDialog(self,
@@ -934,6 +874,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         dialog.set_icon_name("pitivi")
         dialog.set_title(_("Error Loading File"))
         dialog.set_property("secondary-text", unquote(str(exception)))
+        dialog.set_transient_for(self)
         dialog.run()
         dialog.destroy()
         self.set_sensitive(True)
@@ -945,13 +886,21 @@ class PitiviMainWindow(gtk.Window, Loggable):
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
             gtk.STOCK_OPEN, gtk.RESPONSE_OK))
         dialog.set_icon_name("pitivi")
-        dialog.set_border_width(SPACING*2)
+        dialog.set_border_width(SPACING * 2)
         dialog.get_content_area().set_spacing(SPACING)
+        dialog.set_transient_for(self)
 
-        text = _('The following file has moved: "<b>%s</b>" (duration: %s)\
-                \nPlease specify its new location:' \
-                % (factory_name(factory), beautify_length(factory.duration)))
         # TODO: display the filesize to help the user identify the file
+        if not factory.duration or factory.duration == gst.CLOCK_TIME_NONE:
+            # The file is probably an image, not video or audio.
+            text = _('The following file has moved: "<b>%s</b>"'
+                     '\nPlease specify its new location:'
+                     % factory_name(factory))
+        else:
+            length = beautify_length(factory.duration)
+            text = _('The following file has moved: "<b>%s</b>" (duration: %s)'
+                     '\nPlease specify its new location:'
+                     % (factory_name(factory), length))
 
         label = gtk.Label()
         label.set_markup(text)
@@ -1120,7 +1069,9 @@ class PitiviMainWindow(gtk.Window, Loggable):
         chooser.add_filter(default)
 
         response = chooser.run()
-        self.settings.lastProjectFolder = chooser.get_current_folder()
+        current_folder = chooser.get_current_folder()
+        if current_folder:
+            self.settings.lastProjectFolder = current_folder
 
         if response == gtk.RESPONSE_OK:
             self.log("User chose a URI to save project to")
@@ -1175,7 +1126,7 @@ class PitiviMainWindow(gtk.Window, Loggable):
         self.viewer.play()
 
     def _timelineSeekCb(self, ruler, position, format):
-        self.debug("position:%s", gst.TIME_ARGS (position))
+        self.debug("position:%s", gst.TIME_ARGS(position))
         if self.viewer.action != self.project.view_action:
             self.viewer.setPipeline(None)
             self.viewer.hideSlider()
