@@ -16,37 +16,40 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Dialog box for user preferences.
 """
 
 import gtk
-from gettext import gettext as _
+import os
+
 import pitivi.ui.dynamic as dynamic
+
+from pitivi.configure import get_ui_dir
 from pitivi.settings import GlobalSettings
 from pitivi.ui.common import SPACING
+from gettext import gettext as _
 
 GlobalSettings.addConfigOption('prefsDialogWidth',
-    section = "user-interface",
-    key = "prefs-dialog-width",
-    default = 600)
+    section="user-interface",
+    key="prefs-dialog-width",
+    default=600)
 
 GlobalSettings.addConfigOption('prefsDialogHeight',
-    section = "user-interface",
-    key = "prefs-dialog-height",
-    default = 400)
+    section="user-interface",
+    key="prefs-dialog-height",
+    default=400)
 
-class PreferencesDialog(gtk.Window):
+
+class PreferencesDialog():
 
     prefs = {}
     original_values = {}
 
     def __init__(self, instance):
-        gtk.Window.__init__(self)
-        self.app = instance
         self.settings = instance.settings
         self.widgets = {}
         self.resets = {}
@@ -56,76 +59,24 @@ class PreferencesDialog(gtk.Window):
         min_width, min_height = self.contents.size_request()
         width = max(min_width, self.settings.prefsDialogWidth)
         height = max(min_height, self.settings.prefsDialogHeight)
-        self.set_default_size(width, height)
+        self.dialog.set_default_size(width, height)
+
+    def run(self):
+        self.dialog.run()
 
     def _createUi(self):
-        self.set_title(_("Preferences"))
-        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
-        self.set_border_width(12)
-        self.connect("configure-event", self._configureCb)
+        builder = gtk.Builder()
+        builder.add_from_file(os.path.join(get_ui_dir(), "preferences.ui"))
+        builder.connect_signals(self)
 
-        # basic layout
-        vbox = gtk.VBox()
-        vbox.set_spacing(SPACING)
-        button_box = gtk.HBox()
-        button_box.set_spacing(SPACING)
-        button_box.set_homogeneous(False)
-        pane = gtk.HPaned()
-        vbox.pack_start(pane, True, True)
-        vbox.pack_end(button_box, False, False)
-        pane.show()
-        self.add(vbox)
-        vbox.show()
-
-        # left-side list view
-        self.model = gtk.ListStore(str, str)
-        self.treeview = gtk.TreeView(self.model)
-        self.treeview.get_selection().connect("changed",
-            self._treeSelectionChangedCb)
-        ren = gtk.CellRendererText()
-        col = gtk.TreeViewColumn(_("Section"), ren, text=0)
-        self.treeview.append_column(col)
-        scrolled = gtk.ScrolledWindow()
-        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        scrolled.add(self.treeview)
-        scrolled.set_shadow_type(gtk.SHADOW_ETCHED_IN)
-        self.treeview.props.headers_visible = False
-        self.treeview.show()
-        pane.pack1(scrolled)
-        scrolled.show()
-
-        # preferences content region
-        self.contents = gtk.VBox()
-        scrolled = gtk.ScrolledWindow()
-        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        scrolled.add_with_viewport(self.contents)
-        pane.pack2(scrolled)
-        scrolled.show()
-        self.contents.show()
-
-        # revert, close buttons
-        self.factory_settings = gtk.Button(label=_("Reset to Factory Settings"))
-        self.factory_settings.connect("clicked", self._factorySettingsButtonCb)
-        self.factory_settings.set_sensitive(self._canReset())
-        self.factory_settings.show()
-        self.revert_button = gtk.Button(_("Revert"))
-        self.revert_button.connect("clicked", self._revertButtonCb)
-        self.revert_button.show()
-        self.revert_button.set_sensitive(False)
-        accept_button = gtk.Button(stock=gtk.STOCK_CLOSE)
-        accept_button.connect("clicked", self._acceptButtonCb)
-        accept_button.show()
-        button_box.pack_start(self.factory_settings, False, True)
-        button_box.pack_end(accept_button, False, True)
-        button_box.pack_end(self.revert_button, False, True)
-        button_box.show()
-
-        # restart warning
-        self.restart_warning = gtk.Label()
-        self.restart_warning.set_markup(
-            "<b>%s</b>" % _("Some changes will not take effect until you "
-            "restart PiTiVi"))
-        vbox.pack_end(self.restart_warning, False, False)
+        # widgets we'll need
+        self.dialog = builder.get_object("dialog1")
+        self.model = builder.get_object("liststore1")
+        self.treeview = builder.get_object("treeview1")
+        self.contents = builder.get_object("box1")
+        self.revert_button = builder.get_object("revertButton")
+        self.factory_settings = builder.get_object("resetButton")
+        self.restart_warning = builder.get_object("restartWarning")
 
 ## Public API
 
@@ -174,7 +125,7 @@ class PreferencesDialog(gtk.Window):
 
     @classmethod
     def addNumericPreference(cls, attrname, label, description, section=None,
-        upper = None, lower = None):
+        upper=None, lower=None):
         """
         Add an auto-generated user preference that will show up as either a
         gtk.SpinButton or a gtk.HScale, depending whether both the upper and lower
@@ -198,7 +149,7 @@ class PreferencesDialog(gtk.Window):
 
     @classmethod
     def addTextPreference(cls, attrname, label, description, section=None,
-        matches = None):
+        matches=None):
         """
         Add an auto-generated user preference that will show up as either a
         gtk.SpinButton or a gtk.HScale, depending on the upper and lower
@@ -295,9 +246,7 @@ class PreferencesDialog(gtk.Window):
         cls.addPreference(attrname, label, description, section,
             dynamic.FontWidget)
 
-
 ## Implementation
-
     def _fillContents(self):
         self.sections = {}
         for section in sorted(self.prefs):
@@ -320,7 +269,7 @@ class PreferencesDialog(gtk.Window):
                 self.widgets[attrname] = widget
                 revert = gtk.Button(_("Reset"))
                 revert.set_sensitive(not self.settings.isDefault(attrname))
-                revert.connect("clicked",  self._resetOptionCb, attrname)
+                revert.connect("clicked", self._resetOptionCb, attrname)
                 self.resets[attrname] = revert
                 prefs[label] = (label_widget, widget, revert)
 
@@ -377,7 +326,7 @@ class PreferencesDialog(gtk.Window):
 
     def _acceptButtonCb(self, unused_button):
         self._clearHistory()
-        self.hide()
+        self.dialog.hide()
 
     def _valueChanged(self, fake_widget, real_widget, attrname):
         value = getattr(self.settings, attrname)
@@ -432,54 +381,53 @@ if False:
 ## Numeric
 
     PreferencesDialog.addNumericPreference('numericPreference1',
-        label = "Open Range",
-        section = "Test",
-        description = "This option has no upper bound",
-        lower = -10)
+        label="Open Range",
+        section="Test",
+        description="This option has no upper bound",
+        lower=-10)
 
     PreferencesDialog.addNumericPreference('numericPreference2',
-        label = "Closed Range",
-        section = "Test",
-        description = "This option has both upper and lower bounds",
-        lower = -10,
-        upper = 10000)
+        label="Closed Range",
+        section="Test",
+        description="This option has both upper and lower bounds",
+        lower=-10,
+        upper=10000)
 
 ## Text
 
     PreferencesDialog.addTextPreference('textPreference1',
-        label = "Unfiltered",
-        section = "Test",
-        description = "Anything can go in this box")
+        label="Unfiltered",
+        section="Test",
+        description="Anything can go in this box")
 
     PreferencesDialog.addTextPreference('textPreference2',
-        label = "Numbers only",
-        section = "Test",
-        description = "This input validates its input with a regex",
-        matches = "^-?\d+(\.\d+)?$")
+        label="Numbers only",
+        section="Test",
+        description="This input validates its input with a regex",
+        matches="^-?\d+(\.\d+)?$")
 
 ## other
-
     PreferencesDialog.addPathPreference('aPathPreference',
-        label = "Test Path",
-        section = "Test",
-        description = "Test the path widget")
+        label="Test Path",
+        section="Test",
+        description="Test the path widget")
 
     PreferencesDialog.addChoicePreference('aChoicePreference',
-        label = "Swallow Velocity",
-        section = "Test",
-        description = "What is the velocity of an african swollow laden " \
+        label="Swallow Velocity",
+        section="Test",
+        description="What is the velocity of an african swollow laden " \
             "a coconut?",
-        choices = (
+        choices=(
             ("42 Knots", 32),
             ("9 furlongs per fortnight", 42),
             ("I don't know that!", None)))
 
     PreferencesDialog.addChoicePreference('aLongChoicePreference',
-        label = "Favorite Color",
-        section = "Test",
-        description = "What is the velocity of an african swollow laden " \
+        label="Favorite Color",
+        section="Test",
+        description="What is the velocity of an african swollow laden " \
             "a coconut?",
-        choices = (
+        choices=(
             ("Mauve", "Mauve"),
             ("Chartreuse", "Chartreuse"),
             ("Magenta", "Magenta"),
@@ -488,11 +436,11 @@ if False:
             ("Yellow Ochre", "Yellow Ochre")))
 
     PreferencesDialog.addTogglePreference('aTogglePreference',
-        label = "Test Toggle",
-        section = "Test",
-        description = "Test the toggle widget")
+        label="Test Toggle",
+        section="Test",
+        description="Test the toggle widget")
 
     PreferencesDialog.addFontPreference('aFontPreference',
-        label = "Foo Font",
-        section = "Test",
-        description = "Test the font widget")
+        label="Foo Font",
+        section="Test",
+        description="Test the font widget")
