@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # PiTiVi , Non-linear video editor
 #
 #       previewer.py
@@ -17,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Utility tools and classes for easy generation of previews
@@ -66,34 +65,34 @@ GlobalSettings.addConfigOption("thumbnailCacheSize",
     key="cache-size",
     default=250)
 
-# the maximum number of thumbnails to enqueue at a given time. setting this to 
+# the maximum number of thumbnails to enqueue at a given time. setting this to
 # a larger value will increase latency after large operations, such as zooming
 GlobalSettings.addConfigOption("thumbnailMaxRequests",
     section="thumbnailing",
     key="max-requests",
-    default = 10)
+    default=10)
 
 GlobalSettings.addConfigOption('showThumbnails',
-    section = 'user-interface',
-    key = 'show-thumbnails',
-    default = True,
-    notify = True)
+    section='user-interface',
+    key='show-thumbnails',
+    default=True,
+    notify=True)
 
 PreferencesDialog.addTogglePreference('showThumbnails',
-    section = _("Appearance"),
-    label = _("Show Thumbnails (Video)"),
-    description = _("Show Thumbnails on Video Clips"))
+    section=_("Appearance"),
+    label=_("Show Thumbnails (Video)"),
+    description=_("Show Thumbnails on Video Clips"))
 
 GlobalSettings.addConfigOption('showWaveforms',
-    section = 'user-interface',
-    key = 'show-waveforms',
-    default = True,
-    notify = True)
+    section='user-interface',
+    key='show-waveforms',
+    default=True,
+    notify=True)
 
 PreferencesDialog.addTogglePreference('showWaveforms',
-    section = _("Appearance"),
-    label = _("Show Waveforms (Audio)"),
-    description = _("Show Waveforms on Audio Clips"))
+    section=_("Appearance"),
+    label=_("Show Waveforms (Audio)"),
+    description=_("Show Waveforms on Audio Clips"))
 
 # Previewer                      -- abstract base class with public interface for UI
 # |_DefaultPreviewer             -- draws a default thumbnail for UI
@@ -107,14 +106,10 @@ PreferencesDialog.addTogglePreference('showWaveforms',
 
 previewers = {}
 
+
 def get_preview_for_object(instance, trackobject):
     factory = trackobject.factory
-    for stream_ in factory.getOutputStreams():
-        if stream_.isCompatible(trackobject.track.stream):
-            break
-        stream_ = None
-    if not stream_:
-        raise NotImplementedError
+    stream_ = trackobject.stream
     stream_type = type(stream_)
     key = factory, stream_
     if not key in previewers:
@@ -133,10 +128,11 @@ def get_preview_for_object(instance, trackobject):
             previewers[key] = DefaultPreviewer(instance, factory, stream_)
     return previewers[key]
 
+
 class Previewer(Signallable, Loggable):
 
     __signals__ = {
-        "update" : ("segment",),
+        "update": ("segment",),
     }
 
     # TODO: parameterize height, instead of assuming self.theight pixels.
@@ -154,7 +150,7 @@ class Previewer(Signallable, Loggable):
         self.default_thumb = cairo.ImageSurface.create_from_png(path)
         self._connectSettings(instance.settings)
 
-    def render_cairo(self, cr, bounds, element, y1):
+    def render_cairo(self, cr, bounds, element, hscroll_pos, y1):
         """Render a preview of element onto a cairo context within the current
         bounds, which may or may not be the entire object and which may or may
         not intersect the visible portion of the object"""
@@ -163,14 +159,15 @@ class Previewer(Signallable, Loggable):
     def _connectSettings(self, settings):
         self._settings = settings
 
+
 class DefaultPreviewer(Previewer):
 
-    def render_cairo(self, cr, bounds, element, y1):
+    def render_cairo(self, cr, bounds, element, hscroll_pos, y1):
         # TODO: draw a single thumbnail
         pass
 
-class RandomAccessPreviewer(Previewer):
 
+class RandomAccessPreviewer(Previewer):
     """ Handles loading, caching, and drawing preview data for segments of
     random-access streams.  There is one Previewer per stream per
     ObjectFactory.  Preview data is read from an instance of an
@@ -207,7 +204,7 @@ class RandomAccessPreviewer(Previewer):
 
 ## public interface
 
-    def render_cairo(self, cr, bounds, element, y1):
+    def render_cairo(self, cr, bounds, element, hscroll_pos, y1):
         if not self._view:
             return
         # The idea is to conceptually divide the clip into a sequence of
@@ -228,8 +225,9 @@ class RandomAccessPreviewer(Previewer):
 
         # tdur = duration in ns of thumbnail
         # sof  = start of file in pixel coordinates
-        x1 = bounds.x1;
-        sof = Zoomable.nsToPixel(element.start - element.in_point)
+        x1 = bounds.x1
+        sof = Zoomable.nsToPixel(element.start - element.in_point) +\
+            hscroll_pos
 
         # i = left edge of thumbnail to be drawn. We start with x1 and
         # subtract the distance to the nearest leftward rectangle.
@@ -342,6 +340,7 @@ class RandomAccessPreviewer(Previewer):
         self.spacing = settings.thumbnailSpacingHint
         self.emit("update", None)
 
+
 class RandomAccessVideoPreviewer(RandomAccessPreviewer):
 
     @property
@@ -371,11 +370,11 @@ class RandomAccessVideoPreviewer(RandomAccessPreviewer):
             (self.theight, self.twidth + 2))
         filter_ = utils.filter_(caps)
         self.videopipeline = utils.pipeline({
-            sbin : csp,
-            csp : scale,
-            scale : filter_,
-            filter_ : sink,
-            sink : None
+            sbin: csp,
+            csp: scale,
+            scale: filter_,
+            filter_: sink,
+            sink: None
         })
         sink.connect('thumbnail', self._thumbnailCb)
         self.videopipeline.set_state(gst.STATE_PAUSED)
@@ -403,11 +402,12 @@ class RandomAccessVideoPreviewer(RandomAccessPreviewer):
         self._view = settings.showThumbnails
         self.emit("update", None)
 
-class StillImagePreviewer(RandomAccessVideoPreviewer):
 
+class StillImagePreviewer(RandomAccessVideoPreviewer):
     def _thumbForTime(self, cr, time, x, y):
         return RandomAccessVideoPreviewer._thumbForTime(self, cr, 0L, x, y)
 
+
 class RandomAccessAudioPreviewer(RandomAccessPreviewer):
 
     def __init__(self, instance, factory, stream_):
@@ -425,9 +425,9 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
         self.audioSink = ArraySink()
         conv = gst.element_factory_make("audioconvert")
         self.audioPipeline = utils.pipeline({
-            sbin : conv,
-            conv : self.audioSink,
-            self.audioSink : None})
+            sbin: conv,
+            conv: self.audioSink,
+            self.audioSink: None})
         bus = self.audioPipeline.get_bus()
         bus.add_signal_watch()
         bus.connect("message::segment-done", self._busMessageSegmentDoneCb)
@@ -480,7 +480,7 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
                width, self.theight)
             cr = cairo.Context(scaled)
             matrix = cairo.Matrix()
-            matrix.scale(self.base_width/width, 1.0)
+            matrix.scale(self.base_width / width, 1.0)
             cr.set_source_surface(surface)
             cr.get_source().set_matrix(matrix)
             cr.rectangle(0, 0, width, self.theight)
@@ -564,4 +564,3 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
     def _showWaveformsChanged(self, settings):
         self._view = settings.showWaveforms
         self.emit("update", None)
-
