@@ -16,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Widget for the complex view ruler
@@ -31,16 +31,15 @@ from pitivi.ui.zoominterface import Zoomable
 from pitivi.log.loggable import Loggable
 from pitivi.utils import time_to_string
 
-class ScaleRuler(gtk.Layout, Zoomable, Loggable):
+
+class ScaleRuler(gtk.DrawingArea, Zoomable, Loggable):
 
     __gsignals__ = {
-        "expose-event":"override",
-        "size-allocate":"override",
-        "realize":"override",
-        "button-press-event":"override",
-        "button-release-event":"override",
-        "motion-notify-event":"override",
-        "scroll-event" : "override",
+        "expose-event": "override",
+        "button-press-event": "override",
+        "button-release-event": "override",
+        "motion-notify-event": "override",
+        "scroll-event": "override",
         "seek": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                 [gobject.TYPE_UINT64])
         }
@@ -51,15 +50,13 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
     subdivide = ((1, 1.0), (2, 0.5), (10, .25))
 
     def __init__(self, instance, hadj):
-        gtk.Layout.__init__(self)
+        gtk.DrawingArea.__init__(self)
         Zoomable.__init__(self)
         Loggable.__init__(self)
         self.log("Creating new ScaleRule")
         self.add_events(gtk.gdk.POINTER_MOTION_MASK |
             gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK)
-        self.set_hadjustment(hadj)
         self.hadj = hadj
-        self.pixel_position_offset = 0
         hadj.connect("value-changed", self._hadjValueChangedCb)
 
         # double-buffering properties
@@ -79,17 +76,19 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         self.max_duration = gst.CLOCK_TIME_NONE
         self.min_frame_spacing = 5.0
         self.frame_height = 5.0
-        self.frame_rate = gst.Fraction(1/1)
+        self.frame_rate = gst.Fraction(1 / 1)
         self.app = instance
+        self.need_update = True
 
     def _hadjValueChangedCb(self, hadj):
-        self.pixel_position_offset = Zoomable.nsToPixel(self.position) - hadj.get_value()
+        self.pixmap_offset = self.hadj.get_value()
+        self.need_update = True
+        self.queue_draw()
 
 ## Zoomable interface override
 
     def zoomChanged(self):
-        self.queue_resize()
-        self.doPixmap()
+        self.need_update = True
         self.queue_draw()
 
 ## timeline position changed method
@@ -101,47 +100,33 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         npos = max(self.nsToPixel(self.position) - 1, 0)
         self._hadjValueChangedCb(self.hadj)
         height = self.get_allocation().height
-        self.bin_window.invalidate_rect((ppos, 0, 2, height), True)
-        self.bin_window.invalidate_rect((npos, 0, 2, height), True)
+        self.window.invalidate_rect((ppos, 0, 2, height), True)
+        self.window.invalidate_rect((npos, 0, 2, height), True)
 
 ## gtk.Widget overrides
 
-    def do_size_allocate(self, allocation):
-        self.debug("ScaleRuler got %s", list(allocation))
-        gtk.Layout.do_size_allocate(self, allocation)
-        width = max(self.getMaxDurationWidth(), allocation.width)
-        self.debug("Setting layout size to %d x %d",
-                   width, allocation.height)
-        self.set_size(width, allocation.height)
-        new_pos = Zoomable.nsToPixel(self.position) -\
-            self.pixel_position_offset
-        self.hadj.set_value(new_pos)
-        # the size has changed, therefore we want to redo our pixmap
-        self.doPixmap()
-
-    def do_realize(self):
-        gtk.Layout.do_realize(self)
-        # we want to create our own pixmap here
-        self.doPixmap()
-
     def do_expose_event(self, event):
         self.debug("exposing ScaleRuler %s", list(event.area))
         x, y, width, height = event.area
-        if (x < self.pixmap_offset) or (x+width > self.pixmap_offset + self.pixmap_allocated_width):
-            self.debug("exposing outside boundaries !")
-            self.pixmap_offset = max(0, x + (width / 2) - (self.pixmap_allocated_width / 2))
-            self.debug("offset is now %d", self.pixmap_offset)
+        # if (x < self.pixmap_offset) or (x+width > self.pixmap_offset + self.pixmap_allocated_width):
+        #     self.debug("exposing outside boundaries !")
+        #     self.pixmap_offset = max(0, x + (width / 2) - (self.pixmap_allocated_width / 2))
+        #     self.debug("offset is now %d", self.pixmap_offset)
+        #     self.doPixmap()
+        #     width = self.pixmap_allocated_width
+
+        if self.need_update:
             self.doPixmap()
-            width = self.pixmap_allocated_width
+            self.need_update = False
 
         # double buffering power !
-        self.bin_window.draw_drawable(
+        self.window.draw_drawable(
             self.style.fg_gc[gtk.STATE_NORMAL],
             self.pixmap,
-            x - self.pixmap_offset, y,
+            x, y,
             x, y, width, height)
         # draw the position
-        context = self.bin_window.cairo_create()
+        context = self.window.cairo_create()
         self.drawPosition(context, self.get_allocation())
         return False
 
@@ -151,7 +136,7 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
             self.debug("no timeline to seek on, ignoring")
         self.pressed = True
         # seek at position
-        cur = self.pixelToNs(event.x)
+        cur = self.pixelToNs(event.x + self.pixmap_offset)
         self._doSeek(cur)
         return True
 
@@ -164,7 +149,7 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         self.debug("motion at event.x %d", event.x)
         if self.pressed:
             # seek at position
-            cur = self.pixelToNs(event.x)
+            cur = self.pixelToNs(event.x + self.pixmap_offset)
             self._doSeek(cur)
         return False
 
@@ -215,17 +200,11 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         # width (allocation.width)
 
         allocation = self.get_allocation()
-        lwidth, lheight = self.get_size()
-
-        self.pixmap_visible_width = allocation.width
-        self.pixmap_allocated_width = self.pixmap_visible_width * self.pixmap_multiples
-        allocation.width = self.pixmap_allocated_width
-
 
         if (allocation.width != self.pixmap_old_allocated_width):
             if self.pixmap:
                 del self.pixmap
-            self.pixmap = gtk.gdk.Pixmap(self.bin_window, allocation.width,
+            self.pixmap = gtk.gdk.Pixmap(self.window, allocation.width,
                                          allocation.height)
             self.pixmap_old_allocated_width = allocation.width
 
@@ -239,11 +218,9 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         self.scale[0] = float(2 / rate)
         self.scale[1] = float(5 / rate)
         self.scale[2] = float(10 / rate)
-        self.queue_resize()
 
     def setShadedDuration(self, duration):
         self.info("start/duration changed")
-        self.queue_resize()
 
         self.shaded_duration = duration
 
@@ -251,6 +228,8 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
             position = duration - gst.NSECOND
         else:
             position = self.position
+        self.need_update = True
+        self.queue_draw()
 
     def getShadedDuration(self):
         return self.shaded_duration
@@ -259,18 +238,8 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         return self.nsToPixel(self.getShadedDuration())
 
     def setMaxDuration(self, duration):
-        self.queue_resize()
         self.max_duration = duration
 
-    def getMaxDuration(self):
-        return self.max_duration
-
-    def getMaxDurationWidth(self):
-        return self.nsToPixel(self.getMaxDuration())
-
-    def getPixelPosition(self):
-        return 0
-
     def drawBackground(self, allocation):
         self.pixmap.draw_rectangle(
             self.style.bg_gc[gtk.STATE_NORMAL],
@@ -284,8 +253,8 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
                 self.style.bg_gc[gtk.STATE_ACTIVE],
                 True,
                 0, 0,
-                offset,
-                allocation.height)
+                int(offset),
+                int(allocation.height))
 
     def drawRuler(self, allocation):
         layout = self.create_pango_layout(time_to_string(0))
@@ -317,9 +286,8 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
             dur = scale / float(subdivide)
             if spc < self.min_tick_spacing:
                 break
-            paintpos = float(self.border) + 0.5
-            if offset > 0:
-                paintpos += spacing - offset
+            paintpos = -spacing + 0.5
+            paintpos += spacing - offset
             while paintpos < allocation.width:
                 self.drawTick(allocation, paintpos, height)
                 paintpos += spc
@@ -352,7 +320,7 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         frame_width = self.nsToPixel(ns_per_frame)
         if frame_width >= self.min_frame_spacing:
             offset = self.pixmap_offset % frame_width
-            paintpos = float(self.border) + 0.5
+            paintpos = -frame_width + 0.5
             height = allocation.height
             y = int(height - self.frame_height)
             states = [gtk.STATE_ACTIVE, gtk.STATE_PRELIGHT]
@@ -370,7 +338,8 @@ class ScaleRuler(gtk.Layout, Zoomable, Loggable):
         if self.getShadedDuration() <= 0:
             return
         # a simple RED line will do for now
-        xpos = self.nsToPixel(self.position) + self.border
+        xpos = self.nsToPixel(self.position) + self.border -\
+            self.pixmap_offset
         context.save()
         context.set_line_width(1.5)
         context.set_source_rgb(1.0, 0, 0)
