@@ -16,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Timeline widgets for the complex view
@@ -37,18 +37,29 @@ from timelinecanvas import TimelineCanvas
 from timelinecontrols import TimelineControls
 from pitivi.receiver import receiver, handler
 from zoominterface import Zoomable
-from pitivi.ui.common import LAYER_HEIGHT_EXPANDED, LAYER_SPACING
-from pitivi.timeline.timeline import MoveContext
+from pitivi.ui.common import LAYER_HEIGHT_EXPANDED, LAYER_SPACING, TRACK_SPACING
+from pitivi.timeline.timeline import MoveContext, SELECT
 from pitivi.utils import Seeker
 from pitivi.ui.filelisterrordialog import FileListErrorDialog
 from pitivi.ui.curve import Curve
+from pitivi.ui.common import SPACING
+
+from pitivi.factories.operation import EffectFactory
+
+DND_EFFECT_LIST = [[dnd.VIDEO_EFFECT_TUPLE[0], dnd.EFFECT_TUPLE[0]],\
+                  [dnd.AUDIO_EFFECT_TUPLE[0], dnd.EFFECT_TUPLE[0]]]
+VIDEO_EFFECT_LIST = [dnd.VIDEO_EFFECT_TUPLE[0], dnd.EFFECT_TUPLE[0]],
+AUDIO_EFFECT_LIST = [dnd.AUDIO_EFFECT_TUPLE[0], dnd.EFFECT_TUPLE[0]],
 
 # tooltip text for toolbar
 DELETE = _("Delete Selected")
 SPLIT = _("Split clip at playhead position")
-KEYFRAME = _("Create a keyframe")
-ZOOM_IN =  _("Zoom In")
-ZOOM_OUT =  _("Zoom Out")
+KEYFRAME = _("Add a keyframe")
+PREVFRAME = _("Move to the previous keyframe")
+NEXTFRAME = _("Move to the next keyframe")
+ZOOM_IN = _("Zoom In")
+ZOOM_OUT = _("Zoom Out")
+ZOOM_FIT = _("Zoom Fit")
 UNLINK = _("Break links between clips")
 LINK = _("Link together arbitrary clips")
 UNGROUP = _("Ungroup clips")
@@ -63,6 +74,7 @@ ui = '''
             <placeholder name="Timeline">
                 <menuitem action="ZoomIn" />
                 <menuitem action="ZoomOut" />
+                <menuitem action="ZoomFit" />
             </placeholder>
         </menu>
         <menu action="Timeline">
@@ -75,6 +87,9 @@ ui = '''
                 <menuitem action="UnlinkObj" />
                 <menuitem action="GroupObj" />
                 <menuitem action="UngroupObj" />
+                <separator />
+                <menuitem action="Prevframe" />
+                <menuitem action="Nextframe" />
             </placeholder>
         </menu>
     </menubar>
@@ -117,6 +132,7 @@ ui = '''
 #    |
 #    +--Status Bar ??
 
+
 class InfoStub(gtk.HBox, Loggable):
     """
     Box used to display information on the current state of the timeline
@@ -127,17 +143,17 @@ class InfoStub(gtk.HBox, Loggable):
         Loggable.__init__(self)
         self.errors = []
         self.showing = False
-        self._errorsmessage = _("One or more GStreamer errors has occured!")
+        self._scroll_pos_ns = 0
+        self._errorsmessage = _("One or more GStreamer errors occured!")
         self._makeUI()
 
     def _makeUI(self):
-        self.set_spacing(6)
+        self.set_spacing(SPACING)
         self.erroricon = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
                                                   gtk.ICON_SIZE_SMALL_TOOLBAR)
 
         self.pack_start(self.erroricon, expand=False)
 
-
         self.infolabel = gtk.Label(self._errorsmessage)
         self.infolabel.set_alignment(0, 0.5)
 
@@ -184,6 +200,7 @@ class InfoStub(gtk.HBox, Loggable):
         gtk.VBox.hide(self)
         self.showing = False
 
+
 class Timeline(gtk.Table, Loggable, Zoomable):
 
     # the screen width of the current unit
@@ -191,7 +208,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
     # specific levels of zoom, in (multiplier, unit) pairs which
     # from zoomed out to zoomed in
 
-
     def __init__(self, instance, ui_manager):
         gtk.Table.__init__(self, rows=2, columns=1, homogeneous=False)
         Loggable.__init__(self)
@@ -209,8 +225,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._state = gst.STATE_NULL
         self._createUI()
         self._prev_duration = 0
-        self.shrink = True
-        self.rate = gst.Fraction(1,1)
+        self.rate = gst.Fraction(1, 1)
 
     def _createUI(self):
         self.leftSizeGroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -219,6 +234,14 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.hadj = gtk.Adjustment()
         self.vadj = gtk.Adjustment()
 
+        # zooming slider's "zoom fit" button
+        zoom_controls_hbox = gtk.HBox()
+        zoom_best_fit_button = gtk.Button(_("Zoom"))
+        zoom_best_fit_button.set_relief(gtk.RELIEF_NONE)
+        zoom_best_fit_button.set_tooltip_text(ZOOM_FIT)
+        zoom_best_fit_button.set_image(gtk.image_new_from_stock(gtk.STOCK_ZOOM_FIT, gtk.ICON_SIZE_BUTTON))
+        zoom_best_fit_button.connect("clicked", self._zoomFitCb)
+        zoom_controls_hbox.pack_start(zoom_best_fit_button)
         # zooming slider
         self._zoomAdjustment = gtk.Adjustment()
         self._zoomAdjustment.set_value(Zoomable.getCurrentZoomLevel())
@@ -229,7 +252,10 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         zoomslider = gtk.HScale(self._zoomAdjustment)
         zoomslider.props.draw_value = False
         zoomslider.set_tooltip_text(_("Zoom Timeline"))
-        self.attach(zoomslider, 0, 1, 0, 1, yoptions=0, xoptions=gtk.FILL)
+        zoomslider.connect("scroll-event", self._zoomSliderScrollCb)
+        zoomslider.set_size_request(100, 0)  # At least 100px wide for precision
+        zoom_controls_hbox.pack_start(zoomslider)
+        self.attach(zoom_controls_hbox, 0, 1, 0, 1, yoptions=0, xoptions=gtk.FILL)
 
         # controls for tracks and layers
         self._controls = TimelineControls()
@@ -237,12 +263,12 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         controlwindow.add(self._controls)
         controlwindow.set_size_request(-1, 1)
         controlwindow.set_shadow_type(gtk.SHADOW_OUT)
-        self.attach(controlwindow, 0, 1, 1, 2, xoptions=0)
+        self.attach(controlwindow, 0, 1, 1, 2, xoptions=gtk.FILL)
 
         # timeline ruler
         self.ruler = ruler.ScaleRuler(self.app, self.hadj)
         self.ruler.set_size_request(0, 25)
-        self.ruler.set_border_width(2)
+        #self.ruler.set_border_width(2)
         self.ruler.connect("key-press-event", self._keyPressEventCb)
         self.ruler.connect("size-allocate", self._rulerSizeAllocateCb)
         rulerframe = gtk.Frame()
@@ -252,27 +278,21 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
         # proportional timeline
         self._canvas = TimelineCanvas(self.app)
-        timelinewindow = gtk.ScrolledWindow(self.hadj, self.vadj)
-        timelinewindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        timelinewindow.add(self._canvas)
-        timelinewindow.set_shadow_type(gtk.SHADOW_IN)
-        timelinewindow.set_name("timelinewindow")
+        self._root_item = self._canvas.get_root_item()
+        self.attach(self._canvas, 1, 2, 1, 2)
+        self.vadj.connect("changed", self._unsureVadjHeightCb)
 
-        # temp fix for padding between scrollbar and scrolled window
-        #FIXME: should be set at an global position for easy editing?
-        gtk.rc_parse_string("""
-            style 'timelinewindow'
-            {
-                GtkScrolledWindow::scrollbar-spacing = 0
-            }
-            widget '*.timelinewindow' style 'timelinewindow'
-        """)
-
-        self.attach(timelinewindow, 1, 2, 1, 2)
+        # scrollbar
+        self._hscrollbar = gtk.HScrollbar(self.hadj)
+        self._vscrollbar = gtk.VScrollbar(self.vadj)
+        self.attach(self._hscrollbar, 1, 2, 2, 3, yoptions=0)
+        self.attach(self._vscrollbar, 2, 3, 1, 2, xoptions=0)
+        self.hadj.connect("value-changed", self._updateScrollPosition)
+        self.vadj.connect("value-changed", self._updateScrollPosition)
 
         # error infostub
         self.infostub = InfoStub()
-        self.attach(self.infostub, 1, 2, 2, 3, yoptions=0)
+        self.attach(self.infostub, 1, 2, 4, 5, yoptions=0)
 
         self.show_all()
         self.infostub.hide()
@@ -283,6 +303,8 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                 self._zoomInCb),
             ("ZoomOut", gtk.STOCK_ZOOM_OUT, None, "<Control>minus", ZOOM_OUT,
                 self._zoomOutCb),
+            ("ZoomFit", gtk.STOCK_ZOOM_FIT, None, None, ZOOM_FIT,
+                self._zoomFitCb),
 
             # actions for adding additional accelerators
             ("ControlEqualAccel", gtk.STOCK_ZOOM_IN, None, "<Control>equal", ZOOM_IN,
@@ -306,11 +328,15 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                 self.groupSelected),
         )
 
-        playhead_actions = (
+        self.playhead_actions = (
             ("Split", "pitivi-split", _("Split"), "S", SPLIT,
                 self.split),
-            ("Keyframe", "pitivi-keyframe", _("Keyframe"), "K", KEYFRAME,
+            ("Keyframe", "pitivi-keyframe", _("Add a Keyframe"), "K", KEYFRAME,
                 self.keyframe),
+            ("Prevframe", "pitivi-prevframe", _("_Previous Keyframe"), "E", PREVFRAME,
+                self.prevframe),
+            ("Nextframe", "pitivi-nextframe", _("_Next Keyframe"), "R", NEXTFRAME,
+                self.nextframe),
         )
 
         actiongroup = gtk.ActionGroup("timelinepermanent")
@@ -319,7 +345,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
         actiongroup = gtk.ActionGroup("timelineselection")
         actiongroup.add_actions(selection_actions)
-        actiongroup.add_actions(playhead_actions)
+        actiongroup.add_actions(self.playhead_actions)
         self.link_action = actiongroup.get_action("LinkObj")
         self.unlink_action = actiongroup.get_action("UnlinkObj")
         self.group_action = actiongroup.get_action("GroupObj")
@@ -327,27 +353,26 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.delete_action = actiongroup.get_action("DeleteObj")
         self.split_action = actiongroup.get_action("Split")
         self.keyframe_action = actiongroup.get_action("Keyframe")
+        self.prevframe_action = actiongroup.get_action("Prevframe")
+        self.nextframe_action = actiongroup.get_action("Nextframe")
 
         self.ui_manager.insert_action_group(actiongroup, -1)
 
         self.ui_manager.add_ui_from_string(ui)
 
         # drag and drop
-        self.drag_dest_set(gtk.DEST_DEFAULT_MOTION, 
-            [dnd.FILESOURCE_TUPLE],
+        self.drag_dest_set(gtk.DEST_DEFAULT_MOTION,
+            [dnd.FILESOURCE_TUPLE, dnd.EFFECT_TUPLE],
             gtk.gdk.ACTION_COPY)
 
         self.connect("drag-data-received", self._dragDataReceivedCb)
         self.connect("drag-leave", self._dragLeaveCb)
         self.connect("drag-drop", self._dragDropCb)
         self.connect("drag-motion", self._dragMotionCb)
-        self._canvas.connect("button-press-event", self._buttonPress)
-        self._canvas.connect("button-release-event", self._buttonRelease)
         self._canvas.connect("key-press-event", self._keyPressEventCb)
-
+        self._canvas.connect("scroll-event", self._scrollEventCb)
 
 ## Event callbacks
-
     def _keyPressEventCb(self, unused_widget, event):
         kv = event.keyval
         self.debug("kv:%r", kv)
@@ -363,14 +388,14 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                 if mod & gtk.gdk.SHIFT_MASK:
                     self._seekRelative(-gst.SECOND)
                 elif mod & gtk.gdk.CONTROL_MASK:
-                    self._seeker.seek(ltime+1)
+                    self._seeker.seek(ltime + 1)
                 else:
                     self._seekRelative(-long(self.rate * gst.SECOND))
             elif kv == gtk.keysyms.Right:
                 if mod & gtk.gdk.SHIFT_MASK:
                     self._seekRelative(gst.SECOND)
                 elif mod & gtk.gdk.CONTROL_MASK:
-                    self._seeker.seek(rtime+1)
+                    self._seeker.seek(rtime + 1)
                 else:
                     self._seekRelative(long(self.rate * gst.SECOND))
         finally:
@@ -382,61 +407,81 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             pipeline.getDuration()))
         self._seeker.seek(seekvalue)
 
-    def _buttonPress(self, window, event):
-        self.shrink = False
-
-    def _buttonRelease(self, window, event):
-        self.shrink = True
-        self._timelineStartDurationChanged(self.timeline,
-            self.timeline.duration)
-
 ## Drag and Drop callbacks
 
     def _dragMotionCb(self, unused, context, x, y, timestamp):
         self.warning("self._factories:%r, self._temp_objects:%r",
                      not not self._factories,
                      not not self._temp_objects)
+
         if self._factories is None:
-            atom = gtk.gdk.atom_intern(dnd.FILESOURCE_TUPLE[0])
+            if  context.targets in DND_EFFECT_LIST:
+                atom = gtk.gdk.atom_intern(dnd.EFFECT_TUPLE[0])
+            else:
+                atom = gtk.gdk.atom_intern(dnd.FILESOURCE_TUPLE[0])
+
             self.drag_get_data(context, atom, timestamp)
             self.drag_highlight()
         else:
-            # actual drag-and-drop
-            if not self._temp_objects:
-                self.timeline.disableUpdates()
-                self._add_temp_source()
-                focus = self._temp_objects[0]
-                self._move_context = MoveContext(self.timeline,
-                        focus, set(self._temp_objects[1:]))
-            self._move_temp_source(self.hadj.props.value + x, y)
+            if  context.targets not in DND_EFFECT_LIST:
+                if not self._temp_objects:
+                    self.timeline.disableUpdates()
+                    self._add_temp_source()
+                    focus = self._temp_objects[0]
+                    self._move_context = MoveContext(self.timeline,
+                            focus, set(self._temp_objects[1:]))
+                self._move_temp_source(self.hadj.props.value + x, y)
         return True
 
-    def _dragLeaveCb(self, unused_layout, unused_context, unused_tstamp):
+    def _dragLeaveCb(self, unused_layout, context, unused_tstamp):
         if self._temp_objects:
             try:
                 for obj in self._temp_objects:
                     self.timeline.removeTimelineObject(obj, deep=True)
             finally:
                 self._temp_objects = None
+
         self.drag_unhighlight()
         self.timeline.enableUpdates()
 
     def _dragDropCb(self, widget, context, x, y, timestamp):
-        self.app.action_log.begin("add clip")
-        self.timeline.disableUpdates()
-        self._add_temp_source()
-        focus = self._temp_objects[0]
-        self._move_context = MoveContext(self.timeline,
-                focus, set(self._temp_objects[1:]))
-        self._move_temp_source(self.hadj.props.value + x, y)
-        self._move_context.finish()
-        self.timeline.enableUpdates()
-        self.app.action_log.commit()
-        context.drop_finish(True, timestamp)
-        self._factories = None
-        self._temp_objects = None
-        self.app.current.seeker.seek(self._position)
-        return True
+        if  context.targets not in DND_EFFECT_LIST:
+            self.app.action_log.begin("add clip")
+            self.timeline.disableUpdates()
+
+            self._add_temp_source()
+            self.timeline.selection.setSelection(self._temp_objects, SELECT)
+            focus = self._temp_objects[0]
+            self._move_context = MoveContext(self.timeline,
+                                             focus, set(self._temp_objects[1:]))
+            self._move_temp_source(self.hadj.props.value + x, y)
+            self._move_context.finish()
+            self.app.action_log.commit()
+            context.drop_finish(True, timestamp)
+            self._factories = None
+            self._temp_objects = None
+            self.app.current.seeker.seek(self._position)
+
+            return True
+        elif context.targets in DND_EFFECT_LIST:
+            if not self.timeline.timeline_objects:
+                return False
+            factory = self._factories[0]
+            timeline_objs = self._getTimelineObjectUnderMouse(x, y, factory.getInputStreams()[0])
+            if timeline_objs:
+                self.app.action_log.begin("add effect")
+                self.timeline.addEffectFactoryOnObject(factory,
+                                               timeline_objects=timeline_objs)
+                self.app.action_log.commit()
+                self._factories = None
+                self.app.current.seeker.seek(self._position)
+                context.drop_finish(True, timestamp)
+
+                self.timeline.selection.setSelection(timeline_objs, SELECT)
+
+            return True
+
+        return False
 
     def _dragDataReceivedCb(self, unused_layout, context, x, y,
         selection, targetType, timestamp):
@@ -449,15 +494,33 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         # tell current project to import the uri
         # wait for source-added signal, meanwhile ignore dragMotion signals
         # when ready, add factories to the timeline.
-        if targetType != dnd.TYPE_PITIVI_FILESOURCE:
+
+        if targetType not in [dnd.TYPE_PITIVI_FILESOURCE, dnd.TYPE_PITIVI_EFFECT]:
             context.finish(False, False, timestamp)
             return
 
-        uris = selection.data.split("\n")
-        self._factories = [self.project.sources.getUri(uri) for uri in uris]
+        if targetType == dnd.TYPE_PITIVI_FILESOURCE:
+            uris = selection.data.split("\n")
+            self._factories = [self.project.sources.getUri(uri) for uri in uris]
+        else:
+            if not self.timeline.timeline_objects:
+                return False
+            self._factories = [self.app.effects.getFactoryFromName(selection.data)]
+
         context.drag_status(gtk.gdk.ACTION_COPY, timestamp)
         return True
 
+    def _getTimelineObjectUnderMouse(self, x, y, stream):
+        timeline_objs = []
+        items_in_area = self._canvas.getItemsInArea(x, y - 15, x + 1, y - 30)
+        tracks = [obj for obj in items_in_area[0]]
+        track_objects = [obj for obj in items_in_area[1]]
+        for track_object in track_objects:
+            if (type(stream) == type(track_object.stream)):
+                timeline_objs.append(track_object.timeline_object)
+
+        return timeline_objs
+
     def _add_temp_source(self):
         self._temp_objects = [self.timeline.addSourceFactory(factory)
             for factory in self._factories]
@@ -472,16 +535,85 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
 ## Zooming and Scrolling
 
+    def _scrollEventCb(self, canvas, event):
+        if event.state & gtk.gdk.SHIFT_MASK:
+            # shift + scroll => vertical (up/down) scroll
+            if event.direction == gtk.gdk.SCROLL_UP:
+                self.scroll_up()
+            elif event.direction == gtk.gdk.SCROLL_DOWN:
+                self.scroll_down()
+            event.state &= ~gtk.gdk.SHIFT_MASK
+        elif event.state & gtk.gdk.CONTROL_MASK:
+            # zoom + scroll => zooming (up: zoom in)
+            if event.direction == gtk.gdk.SCROLL_UP:
+                Zoomable.zoomIn()
+                return True
+            elif event.direction == gtk.gdk.SCROLL_DOWN:
+                Zoomable.zoomOut()
+                return True
+            return False
+        else:
+            if event.direction == gtk.gdk.SCROLL_UP:
+                self.scroll_left()
+            elif event.direction == gtk.gdk.SCROLL_DOWN:
+                self.scroll_right()
+        return True
+
+    def scroll_left(self):
+        self._hscrollbar.set_value(self._hscrollbar.get_value() -
+            self.hadj.props.page_size ** (2.0 / 3.0))
+
+    def scroll_right(self):
+        self._hscrollbar.set_value(self._hscrollbar.get_value() +
+            self.hadj.props.page_size ** (2.0 / 3.0))
+
+    def scroll_up(self):
+        self._vscrollbar.set_value(self._vscrollbar.get_value() -
+            self.vadj.props.page_size ** (2.0 / 3.0))
+
+    def scroll_down(self):
+        self._vscrollbar.set_value(self._vscrollbar.get_value() +
+            self.vadj.props.page_size ** (2.0 / 3.0))
+
+    def _updateScrollPosition(self, adjustment):
+        self._scroll_pos_ns = Zoomable.pixelToNs(self.hadj.get_value())
+        self._root_item.set_simple_transform(-self.hadj.get_value(),
+            -self.vadj.get_value(), 1.0, 0)
+
     def _zoomAdjustmentChangedCb(self, adjustment):
         # GTK crack
         self._updateZoom = False
         Zoomable.setZoomLevel(int(adjustment.get_value()))
         self._updateZoom = True
 
+    def _unsureVadjHeightCb(self, adj):
+        # GTK crack, without that, at loading a project, the vadj upper
+        # property is reset to be equal as the lower, right after the
+        # trackobjects are added to the timeline (bug: #648714)
+        if self.vadj.props.upper < self._canvas.height:
+            self.vadj.props.upper = self._canvas.height
+
+    _scroll_pos_ns = 0
+
+    def _zoomSliderScrollCb(self, unused_widget, event):
+        value = self._zoomAdjustment.get_value()
+        if event.direction in [gtk.gdk.SCROLL_UP, gtk.gdk.SCROLL_RIGHT]:
+            self._zoomAdjustment.set_value(value + 1)
+        elif event.direction in [gtk.gdk.SCROLL_DOWN, gtk.gdk.SCROLL_LEFT]:
+            self._zoomAdjustment.set_value(value - 1)
+
     def zoomChanged(self):
-        self._canvas.props.redraw_when_scrolled = True
         if self._updateZoom:
             self._zoomAdjustment.set_value(self.getCurrentZoomLevel())
+
+        # the new scroll position should preserve the current horizontal
+        # position of the playhead in the window
+        cur_playhead_offset = self._canvas._playhead.props.x -\
+            self.hadj.props.value
+        new_pos = Zoomable.nsToPixel(self._position) - cur_playhead_offset
+
+        self._updateScrollAdjustments()
+        self._scrollToPosition(new_pos)
         self.ruler.queue_resize()
         self.ruler.queue_draw()
 
@@ -497,8 +629,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
     def scrollToPlayhead(self):
         """
-        Scroll the current position as close to the center of the view
-        as possible (as close as the timeline canvas allows).
+        If the current position is out of the view bouds, then scroll
+        as close to the center of the view as possible or as close as the
+        timeline canvas allows.
         """
         page_size = self.hadj.get_page_size()
 
@@ -517,7 +650,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             self._scrollToPosition(position)
 
     def _scrollToPosition(self, position):
-        self.hadj.set_value(position)
+        self._hscrollbar.set_value(position)
         return False
 
     def _rulerSizeAllocateCb(self, ruler, allocation):
@@ -558,18 +691,20 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
     @handler(timeline, "duration-changed")
     def _timelineStartDurationChanged(self, unused_timeline, duration):
-        if self.shrink:
-            self._prev_duration = duration
-            self.ruler.setMaxDuration(duration + 60 * gst.SECOND)
-            self._canvas.setMaxDuration(duration + 60 * gst.SECOND)
-            self.ruler.setShadedDuration(duration)
-        else:
-            # only resize if new size is larger
-            if duration > self._prev_duration:
-                self._prev_duration = duration
-                self.ruler.setMaxDuration(duration)
-                self._canvas.setMaxDuration(duration)
-                #self.ruler.setShadedDuration(duration)
+        self._prev_duration = duration
+        self.ruler.setMaxDuration(duration + 60 * gst.SECOND)
+        self._canvas.setMaxDuration(duration + 60 * gst.SECOND)
+        self.ruler.setShadedDuration(duration)
+        self._updateScrollAdjustments()
+
+    def _updateScrollAdjustments(self):
+        a = self.get_allocation()
+        size = Zoomable.nsToPixel(self.timeline.duration)
+        self.hadj.props.lower = 0
+        self.hadj.props.upper = size + 200  # why is this necessary???
+        self.hadj.props.page_size = a.width
+        self.hadj.props.page_increment = size * 0.9
+        self.hadj.props.step_increment = size * 0.1
 
     @handler(timeline, "selection-changed")
     def _timelineSelectionChanged(self, timeline):
@@ -580,7 +715,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         ungroup = False
         split = False
         keyframe = False
-        timeline_objects = {}
         if timeline.selection:
             delete = True
             if len(timeline.selection) > 1:
@@ -604,9 +738,11 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                     start = obj.start
                     duration = obj.duration
 
-            split = True
             keyframe = True
 
+        if (len(timeline.timeline_objects) > 0):
+            split = True
+
         self.delete_action.set_sensitive(delete)
         self.link_action.set_sensitive(link)
         self.unlink_action.set_sensitive(unlink)
@@ -615,13 +751,14 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.split_action.set_sensitive(split)
         self.keyframe_action.set_sensitive(keyframe)
 
-
 ## ToolBar callbacks
-
     def hide(self):
         self.actiongroup.set_visible(False)
         gtk.Vbox.hide(self)
 
+    def _zoomFitCb(self, unused_action):
+        self.app.gui.setBestZoomRatio()
+
     def _zoomInCb(self, unused_action):
         Zoomable.zoomIn()
 
@@ -664,11 +801,11 @@ class Timeline(gtk.Table, Loggable, Zoomable):
     def keyframe(self, action):
         timeline_position = self._position
         selected = self.timeline.selection.getSelectedTrackObjs()
-        
+
         for obj in selected:
             keyframe_exists = False
 
-            position_in_obj = timeline_position - obj.start
+            position_in_obj = (timeline_position - obj.start) + obj.in_point
             interpolators = obj.getInterpolators()
             for value in interpolators:
                 interpolator = obj.getInterpolator(value)
@@ -683,3 +820,19 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                     self.app.action_log.begin("add volume point")
                     interpolator.newKeyframe(position_in_obj)
                     self.app.action_log.commit()
+
+    def prevframe(self, action):
+        timeline_position = self._position
+
+        prev_kf = self.timeline.getPrevKeyframe(timeline_position)
+        if prev_kf != None:
+            self._seeker.seek(prev_kf)
+            self.scrollToPlayhead()
+
+    def nextframe(self, action):
+        timeline_position = self._position
+
+        next_kf = self.timeline.getNextKeyframe(timeline_position)
+        if next_kf:
+            self._seeker.seek(next_kf)
+            self.scrollToPlayhead()
