@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # PiTiVi , Non-linear video editor
 #
 #       utils.py
@@ -18,14 +17,15 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 # set of utility functions
 
 import sys
 import gobject
-import gst, bisect
+import gst
+import bisect
 import os
 import struct
 from pitivi.signalinterface import Signallable
@@ -38,9 +38,11 @@ except ImportError:
 
 UNKNOWN_DURATION = 2 ** 63 - 1
 
+
 def between(a, b, c):
     return (a <= b) and (b <= c)
 
+
 def time_to_string(value):
     """
     Converts the given time in nanoseconds to a human readable string
@@ -58,6 +60,7 @@ def time_to_string(value):
     mins = mins % 60
     return "%01d:%02d:%02d.%03d" % (hours, mins, sec, ms)
 
+
 def beautify_length(length):
     """
     Converts the given time in nanoseconds to a human readable string
@@ -80,6 +83,7 @@ def beautify_length(length):
 
     return ", ".join(parts)
 
+
 def beautify_ETA(length):
     """
     Converts the given time in nanoseconds to a fuzzy estimate,
@@ -103,6 +107,7 @@ def beautify_ETA(length):
 
     return ", ".join(parts)
 
+
 def bin_contains(bin, element):
     """ Returns True if the bin contains the given element, the search is recursive """
     if not isinstance(bin, gst.Bin):
@@ -127,21 +132,22 @@ def bin_contains(bin, element):
 # matches the list, this function returns the index of the lement closest to
 # value in the array.
 
+
 def binary_search(col, value):
     low = 0
     high = len(col)
     while (low < high):
-        mid = (low + high)/2
+        mid = (low + high) / 2
         if (col[mid] < value):
             low = mid + 1
         else:
             #can't be high = mid-1: here col[mid] >= value,
             #so high can't be < mid if col[mid] == value
-            high = mid;
+            high = mid
     return low
 
-# Returns the element of seq nearest to item, and the difference between them
 
+# Returns the element of seq nearest to item, and the difference between them
 def closest_item(seq, item, lo=0):
     index = bisect.bisect(seq, item, lo)
     if index >= len(seq):
@@ -163,6 +169,7 @@ def closest_item(seq, item, lo=0):
 
     return res, diff, index
 
+
 def argmax(func, seq):
     """return the element of seq that gives max(map(func, seq))"""
     def compare(a1, b1):
@@ -173,6 +180,7 @@ def argmax(func, seq):
     objs = ((func(val), val) for val in seq)
     return reduce(compare, objs)[1]
 
+
 def same(seq):
     i = iter(seq)
     first = i.next()
@@ -181,6 +189,7 @@ def same(seq):
             return None
     return first
 
+
 def data_probe(pad, data, section=""):
     """Callback to use for gst.Pad.add_*_probe.
 
@@ -189,25 +198,26 @@ def data_probe(pad, data, section=""):
     if section == "":
         section = "%s:%s" % (pad.get_parent().get_name(), pad.get_name())
     if isinstance(data, gst.Buffer):
-        log.debug("probe","%s BUFFER timestamp:%s , duration:%s , size:%d , offset:%d , offset_end:%d",
+        log.debug("probe", "%s BUFFER timestamp:%s , duration:%s , size:%d , offset:%d , offset_end:%d",
                   section, gst.TIME_ARGS(data.timestamp), gst.TIME_ARGS(data.duration),
                   data.size, data.offset, data.offset_end)
         if data.flags & gst.BUFFER_FLAG_DELTA_UNIT:
-            log.debug("probe","%s DELTA_UNIT", section)
+            log.debug("probe", "%s DELTA_UNIT", section)
         if data.flags & gst.BUFFER_FLAG_DISCONT:
-            log.debug("probe","%s DISCONT", section)
+            log.debug("probe", "%s DISCONT", section)
         if data.flags & gst.BUFFER_FLAG_GAP:
-            log.debug("probe","%s GAP", section)
-        log.debug("probe","%s flags:%r", section, data.flags)
+            log.debug("probe", "%s GAP", section)
+        log.debug("probe", "%s flags:%r", section, data.flags)
     else:
-        log.debug("probe","%s EVENT %s", section, data.type)
+        log.debug("probe", "%s EVENT %s", section, data.type)
         if data.type == gst.EVENT_NEWSEGMENT:
             upd, rat, fmt, start, stop, pos = data.parse_new_segment()
-            log.debug("probe","%s Update:%r rate:%f fmt:%s, start:%s, stop:%s, pos:%s",
+            log.debug("probe", "%s Update:%r rate:%f fmt:%s, start:%s, stop:%s, pos:%s",
                       section, upd, rat, fmt, gst.TIME_ARGS(start),
                       gst.TIME_ARGS(stop), gst.TIME_ARGS(pos))
     return True
 
+
 def linkDynamic(element, target):
 
     def pad_added(bin, pad, target):
@@ -216,9 +226,11 @@ def linkDynamic(element, target):
             pad.link_full(compatpad, gst.PAD_LINK_CHECK_NOTHING)
     element.connect("pad-added", pad_added, target)
 
+
 def element_make_many(*args):
     return tuple((gst.element_factory_make(arg) for arg in args))
 
+
 def pipeline(graph):
     E = graph.iteritems()
     V = graph.iterkeys()
@@ -232,6 +244,7 @@ def pipeline(graph):
                 linkDynamic(u, v)
     return p
 
+
 def filter_(caps):
     f = gst.element_factory_make("capsfilter")
     f.props.caps = gst.caps_from_string(caps)
@@ -249,10 +262,10 @@ def uri_is_valid(uri):
     @param uri: The location to check
     @type uri: C{URI}
     """
-    res = gst.uri_is_valid(uri) and gst.uri_get_protocol(uri) == "file"
-    if res:
-        return len(os.path.basename(gst.uri_get_location(uri))) > 0
-    return res
+    return (gst.uri_is_valid(uri) and
+            gst.uri_get_protocol(uri) == "file" and
+            len(os.path.basename(gst.uri_get_location(uri))) > 0)
+
 
 def uri_is_reachable(uri):
     """ Check whether the given uri is reachable and we can read/write
@@ -267,9 +280,10 @@ def uri_is_reachable(uri):
         raise NotImplementedError(
             # Translators: "non local" means the project is not stored
             # on a local filesystem
-            _("%s doesn't yet handle non local projects") % APPNAME)
+            _("%s doesn't yet handle non-local projects") % APPNAME)
     return os.path.isfile(gst.uri_get_location(uri))
 
+
 class PropertyChangeTracker(Signallable):
 
     __signals__ = {}
@@ -305,6 +319,7 @@ class PropertyChangeTracker(Signallable):
 
         self.emit(property_name + '-changed', object, old_value, value)
 
+
 class Seeker(Signallable):
     __signals__ = {'seek': ['position', 'format']}
 
@@ -344,9 +359,11 @@ class Seeker(Signallable):
                 return False
         return False
 
+
 def get_filesystem_encoding():
     return sys.getfilesystemencoding() or "utf-8"
 
+
 def get_controllable_properties(element):
     """
     Returns a list of controllable properties for the given
@@ -368,33 +385,43 @@ def get_controllable_properties(element):
                 res.append((element, prop))
     return res
 
+
 def start_insort_left(a, x, lo=0, hi=None):
     if hi is None:
         hi = len(a)
     while lo < hi:
-        mid = (lo+hi)//2
-        if a[mid].start < x.start: lo = mid+1
-        else: hi = mid
+        mid = (lo + hi) // 2
+        if a[mid].start < x.start:
+            lo = mid + 1
+        else:
+            hi = mid
     a.insert(lo, x)
 
+
 def start_insort_right(a, x, lo=0, hi=None):
     if hi is None:
         hi = len(a)
     while lo < hi:
-        mid = (lo+hi)//2
-        if x.start < a[mid].start: hi = mid
-        else: lo = mid+1
+        mid = (lo + hi) // 2
+        if x.start < a[mid].start:
+            hi = mid
+        else:
+            lo = mid + 1
     a.insert(lo, x)
 
+
 def start_bisect_left(a, x, lo=0, hi=None):
     if hi is None:
         hi = len(a)
     while lo < hi:
-        mid = (lo+hi)//2
-        if a[mid].start < x.start: lo = mid+1
-        else: hi = mid
+        mid = (lo + hi) // 2
+        if a[mid].start < x.start:
+            lo = mid + 1
+        else:
+            hi = mid
     return lo
 
+
 class Infinity(object):
     def __cmp__(self, other):
         if isinstance(other, Infinity):
@@ -404,6 +431,7 @@ class Infinity(object):
 
 infinity = Infinity()
 
+
 def findObject(obj, objects):
     low = 0
     high = len(objects)
@@ -419,6 +447,7 @@ def findObject(obj, objects):
 
     return low
 
+
 def getPreviousObject(obj, objects, priority=-1, skip=None):
     if priority == -1:
         priority = obj.priority
@@ -440,7 +469,6 @@ def getPreviousObject(obj, objects, priority=-1, skip=None):
         if priority is None or prev_obj.priority == priority:
             return prev_obj
 
-
     # check if there are objects with start < obj.start
     prev_obj_index = obj_index - 1
     while prev_obj_index >= 0:
@@ -453,6 +481,7 @@ def getPreviousObject(obj, objects, priority=-1, skip=None):
 
     return None
 
+
 def getNextObject(obj, objects, priority=-1, skip=None):
     if priority == -1:
         priority = obj.priority
@@ -502,6 +531,7 @@ class CachedFactoryList(object):
         log.warning("utils", "New feature added, invalidating cached factories")
         self._factories = None
 
+
 def profile(func, profiler_filename="result.prof"):
     import os.path
     counter = 1
@@ -510,7 +540,7 @@ def profile(func, profiler_filename="result.prof"):
         output_filename = profiler_filename + str(counter)
         counter += 1
 
-    def _wrapper(*args,**kwargs):
+    def _wrapper(*args, **kwargs):
         local_func = func
         cProfile.runctx("result = local_func(*args, **kwargs)", globals(), locals(),
                         filename=output_filename)
@@ -518,6 +548,7 @@ def profile(func, profiler_filename="result.prof"):
 
     return _wrapper
 
+
 def formatPercent(value):
     return "%3d%%" % (value * 100)
 
